@@ -9,10 +9,13 @@
 #include <goffice/goffice-config.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-marshalers.h>
 #include <goffice/app/file.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/app/file-priv.h>
 #include <goffice/app/error-info.h>
 #include <goffice/app/io-context.h>
+#include <goffice/app/go-cmd-context.h>
 
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-output.h>
@@ -58,7 +61,7 @@ go_file_opener_can_probe_real (GOFileOpener const *fo, FileProbeLevel pl)
 
 static gboolean
 go_file_opener_probe_real (GOFileOpener const *fo, GsfInput *input,
-                             FileProbeLevel pl)
+			   FileProbeLevel pl)
 {
 	gboolean ret = FALSE;
 
@@ -70,17 +73,19 @@ go_file_opener_probe_real (GOFileOpener const *fo, GsfInput *input,
 }
 
 static void
-go_file_opener_open_real (GOFileOpener const *fo, gchar const *opt_enc, 
-			   IOContext *io_context,
-			   gpointer FIXME_FIXME_workbook_view,
-			   GsfInput *input)
+go_file_opener_open_real (GOFileOpener const *fo, gchar const *opt_enc,
+			  IOContext *io_context,
+			  gpointer FIXME_workbook_view,
+			  GsfInput *input)
 {
 	if (fo->open_func != NULL) {
 		if (fo->encoding_dependent)
-			((GOFileOpenerOpenFuncWithEnc)fo->open_func) 
-				(fo, opt_enc, io_context, FIXME_FIXME_workbook_view, input);
+			((GOFileOpenerOpenFuncWithEnc)fo->open_func)
+				(fo, opt_enc, io_context,
+				 FIXME_workbook_view, input);
 		else
-			fo->open_func (fo, io_context, FIXME_FIXME_workbook_view, input);
+			fo->open_func (fo, io_context,
+				       FIXME_workbook_view, input);
 	} else
 		gnumeric_io_error_unknown (io_context);
 }
@@ -104,6 +109,8 @@ GSF_CLASS (GOFileOpener, go_file_opener,
  * @fo          : Newly created GOFileOpener object
  * @id          : Optional ID of the opener (or NULL)
  * @description : Description of supported file format
+ * @suffixes	: List of suffixes to associate with the opener
+ * @mimes	: List of mime types to associate with the opener
  * @encoding_dependent: whether the opener depends on an encoding sel.
  * @probe_func  : Optional pointer to "probe" function (or NULL)
  * @open_func   : Pointer to "open" function
@@ -111,15 +118,15 @@ GSF_CLASS (GOFileOpener, go_file_opener,
  * Sets up GOFileOpener object, newly created with g_object_new function.
  * This is intended to be used only by GOFileOpener derivates.
  * Use go_file_opener_new, if you want to create GOFileOpener object.
- */
+ **/
 void
 go_file_opener_setup (GOFileOpener *fo, gchar const *id,
-                        gchar const *description,
-			GSList *suffixes,
-			GSList *mimes,
-		        gboolean encoding_dependent,
-                        GOFileOpenerProbeFunc probe_func,
-                        GOFileOpenerOpenFunc open_func)
+		      gchar const *description,
+		      GSList *suffixes,
+		      GSList *mimes,
+		      gboolean encoding_dependent,
+		      GOFileOpenerProbeFunc probe_func,
+		      GOFileOpenerOpenFunc open_func)
 {
 	g_return_if_fail (IS_GO_FILE_OPENER (fo));
 
@@ -137,27 +144,29 @@ go_file_opener_setup (GOFileOpener *fo, gchar const *id,
  * go_file_opener_new:
  * @id          : Optional ID of the opener (or NULL)
  * @description : Description of supported file format
+ * @suffixes	: List of suffixes to associate with the opener
+ * @mimes	: List of mime types to associate with the opener
  * @probe_func  : Optional pointer to "probe" function (or NULL)
  * @open_func   : Pointer to "open" function
  *
  * Creates new GOFileOpener object. Optional @id will be used
  * after registering it with go_file_opener_register function.
  *
- * Return value: newly created GOFileOpener object.
- */
+ * Returns: newly created GOFileOpener object.
+ **/
 GOFileOpener *
 go_file_opener_new (gchar const *id,
-                      gchar const *description,
-		      GSList *suffixes,
-		      GSList *mimes,
-                      GOFileOpenerProbeFunc probe_func,
-                      GOFileOpenerOpenFunc open_func)
+		    gchar const *description,
+		    GSList *suffixes,
+		    GSList *mimes,
+		    GOFileOpenerProbeFunc probe_func,
+		    GOFileOpenerOpenFunc open_func)
 {
 	GOFileOpener *fo;
 
 	fo = GO_FILE_OPENER (g_object_new (TYPE_GO_FILE_OPENER, NULL));
 	go_file_opener_setup (fo, id, description, suffixes, mimes, FALSE,
-			       probe_func, open_func);
+			      probe_func, open_func);
 
 	return fo;
 }
@@ -165,29 +174,30 @@ go_file_opener_new (gchar const *id,
 /**
  * go_file_opener_new_with_enc:
  * @id          : Optional ID of the opener (or NULL)
- * @description : Description of supported file format 
+ * @description : Description of supported file format
+ * @suffixes	: List of suffixes to associate with the opener
+ * @mimes	: List of mime types to associate with the opener
  * @probe_func  : Optional pointer to "probe" function (or NULL)
- * @open_func   : Pointer to "open" function    
- *   
+ * @open_func   : Pointer to "open" function
+ *
  * Creates new GOFileOpener object. Optional @id will be used
  * after registering it with go_file_opener_register function.
- *      
- * Return value: newly created GOFileOpener object.
- */
-
+ *
+ * Returns: newly created #GOFileOpener object.
+ **/
 GOFileOpener *
 go_file_opener_new_with_enc (gchar const *id,
-		     gchar const *description,
-		     GSList *suffixes,
-		     GSList *mimes,
-		     GOFileOpenerProbeFunc probe_func,
-		     GOFileOpenerOpenFuncWithEnc open_func)
+			     gchar const *description,
+			     GSList *suffixes,
+			     GSList *mimes,
+			     GOFileOpenerProbeFunc probe_func,
+			     GOFileOpenerOpenFuncWithEnc open_func)
 {
         GOFileOpener *fo;
 
         fo = GO_FILE_OPENER (g_object_new (TYPE_GO_FILE_OPENER, NULL));
         go_file_opener_setup (fo, id, description, suffixes, mimes, TRUE,
-                               probe_func, (GOFileOpenerOpenFunc)open_func);
+			      probe_func, (GOFileOpenerOpenFunc)open_func);
         return fo;
 }
 
@@ -210,11 +220,11 @@ go_file_opener_get_description (GOFileOpener const *fo)
 	return fo->description;
 }
 
-gboolean 
+gboolean
 go_file_opener_is_encoding_dependent (GOFileOpener const *fo)
 {
         g_return_val_if_fail (IS_GO_FILE_OPENER (fo), FALSE);
-	
+
 	return fo->encoding_dependent;
 }
 
@@ -242,14 +252,14 @@ go_file_opener_get_mimes (GOFileOpener const *fo)
 
 /**
  * go_file_opener_probe:
- * @fo      : GOFileOpener object
- * @input   : The input source
+ * @fo      : #GOFileOpener
+ * @input   : #GsfInput
+ * @pl	    : #FileProbeLevel
  *
  * Checks if a given file is supported by the opener.
  *
- * Return value: TRUE, if the opener can read given file and FALSE
- *               otherwise.
- */
+ * Returns: %TRUE, if the opener can read given file and %FALSE otherwise.
+ **/
 gboolean
 go_file_opener_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 {
@@ -269,7 +279,7 @@ go_file_opener_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl
  * @fo          : GOFileOpener object
  * @opt_enc     : Optional encoding
  * @io_context  : Context for i/o operation
- * @FIXME_FIXME_workbook_view: Workbook View
+ * @FIXME_workbook_view: Workbook View
  * @input       : Gsf input stream
  *
  * Reads content of @file_name file into workbook @wbv is attached to.
@@ -280,13 +290,14 @@ go_file_opener_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl
  */
 void
 go_file_opener_open (GOFileOpener const *fo, gchar const *opt_enc,
-		      IOContext *io_context,
-		      gpointer FIXME_FIXME_workbook_view, GsfInput *input)
+		     IOContext *io_context,
+		     gpointer FIXME_workbook_view, GsfInput *input)
 {
 	g_return_if_fail (IS_GO_FILE_OPENER (fo));
 	g_return_if_fail (GSF_IS_INPUT (input));
 
-	GO_FILE_OPENER_METHOD (fo, open) (fo, opt_enc, io_context, FIXME_FIXME_workbook_view, input);
+	GO_FILE_OPENER_METHOD (fo, open) (fo, opt_enc, io_context,
+					  FIXME_workbook_view, input);
 }
 
 /*
@@ -333,6 +344,13 @@ enum {
 	FS_PROP_SCOPE
 };
 
+enum {
+	FS_SET_EXPORT_OPTIONS,
+	FS_LAST_SIGNAL
+};
+
+static guint fs_signals[FS_LAST_SIGNAL];
+
 static void
 go_file_saver_set_property (GObject *object, guint property_id,
 			    GValue const *value, GParamSpec *pspec)
@@ -415,14 +433,14 @@ go_file_saver_get_property (GObject *object, guint property_id,
 
 static void
 go_file_saver_save_real (GOFileSaver const *fs, IOContext *io_context,
-			  gconstpointer FIXME_FIXME_workbook_view, GsfOutput *output)
+			 gconstpointer FIXME_workbook_view, GsfOutput *output)
 {
 	if (fs->save_func == NULL) {
 		gnumeric_io_error_unknown (io_context);
 		return;
 	}
 
-	fs->save_func (fs, io_context, FIXME_FIXME_workbook_view, output);
+	fs->save_func (fs, io_context, FIXME_workbook_view, output);
 }
 
 static void
@@ -509,6 +527,17 @@ go_file_saver_class_init (GOFileSaverClass *klass)
 				    G_PARAM_READWRITE));
 
 	klass->save = go_file_saver_save_real;
+
+	/* class signals */
+	fs_signals[FS_SET_EXPORT_OPTIONS] =
+		g_signal_new ("set-export-options",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GOFileSaverClass, set_export_options),
+			      NULL, NULL,
+			      go__BOOLEAN__OBJECT_STRING_POINTER,
+			      G_TYPE_BOOLEAN,
+			      3, GO_DOC_TYPE, G_TYPE_STRING, G_TYPE_POINTER);
 }
 
 GSF_CLASS (GOFileSaver, go_file_saver,
@@ -527,7 +556,7 @@ GSF_CLASS (GOFileSaver, go_file_saver,
  * after registering it with go_file_saver_register or
  * go_file_saver_register_as_default function.
  *
- * Return value: newly created GOFileSaver object.
+ * Returns: newly created GOFileSaver object.
  */
 GOFileSaver *
 go_file_saver_new (gchar const *id,
@@ -606,11 +635,24 @@ go_file_saver_get_format_level (GOFileSaver const *fs)
 	return fs->format_level;
 }
 
+gboolean
+go_file_saver_set_export_options (GOFileSaver *fs,
+				  GODoc *doc,
+				  const char *options,
+				  GError **err)
+{
+	gboolean res = TRUE;
+
+	g_signal_emit (G_OBJECT (fs), fs_signals[FS_SET_EXPORT_OPTIONS], 0,
+		       doc, options, err, &res);
+	return res;
+}
+
 /**
  * go_file_saver_save:
  * @fs          : GOFileSaver object
  * @io_context  : Context for i/o operation
- * @FIXME_FIXME_workbook_view: Workbook View
+ * @FIXME_workbook_view: Workbook View
  * @output      : Output stream
  *
  * Saves @wbv and the workbook it is attached to into @output stream.
@@ -621,8 +663,8 @@ go_file_saver_get_format_level (GOFileSaver const *fs)
  */
 void
 go_file_saver_save (GOFileSaver const *fs, IOContext *io_context,
-		     gconstpointer FIXME_FIXME_workbook_view,
-		     GsfOutput *output)
+		    gconstpointer FIXME_workbook_view,
+		    GsfOutput *output)
 {
 	g_return_if_fail (IS_GO_FILE_SAVER (fs));
 	g_return_if_fail (GSF_IS_OUTPUT (output));
@@ -632,7 +674,7 @@ go_file_saver_save (GOFileSaver const *fs, IOContext *io_context,
 		/*
 		 * This is mostly bogus.  We need a proper source for the file
 		 * name, not something double-converted.
-		 */		   
+		 */
 		char *file_name = name
 			? g_filename_from_utf8 (name, -1, NULL, NULL, NULL)
 			: NULL;
@@ -645,7 +687,7 @@ go_file_saver_save (GOFileSaver const *fs, IOContext *io_context,
 			const char *msg = _("Saving over old files of this type is disabled for safety.");
 
 			if (!gsf_output_error (output))
-				gsf_output_set_error (output, 0, msg);
+				gsf_output_set_error (output, 0, "%s", msg);
 
 			g_free (file_name);
 
@@ -660,7 +702,8 @@ go_file_saver_save (GOFileSaver const *fs, IOContext *io_context,
 		g_free (file_name);
 	}
 
-	GO_FILE_SAVER_METHOD (fs, save) (fs, io_context, FIXME_FIXME_workbook_view, output);
+	GO_FILE_SAVER_METHOD (fs, save) (fs, io_context,
+					 FIXME_workbook_view, output);
 }
 
 /**
@@ -684,36 +727,36 @@ go_file_saver_set_overwrite_files (GOFileSaver *fs, gboolean overwrite)
 GType
 go_file_saver_level_get_type (void)
 {
-  static GType etype = 0;
-  if (etype == 0) {
-	  static GEnumValue values[] = {
-		  { FILE_FL_NONE, (char*)"GO_FILE_SAVER_LEVEL_NONE", (char*)"none" },
-		  { FILE_FL_WRITE_ONLY, (char*)"GO_FILE_SAVER_LEVEL_WRITE_ONLY", (char*)"write_only" },
-		  { FILE_FL_NEW, (char*)"GO_FILE_SAVER_LEVEL_NEW", (char*)"new" },
-		  { FILE_FL_MANUAL, (char*)"GO_FILE_SAVER_LEVEL_MANUAL", (char*)"manual" },
-		  { FILE_FL_MANUAL_REMEMBER, (char*)"GO_FILE_SAVER_LEVEL_MANUAL_REMEMBER", (char*)"manual_remember" },
-		  { FILE_FL_AUTO, (char*)"GO_FILE_SAVER_LEVEL_AUTO", (char*)"auto" },
-		  { 0, NULL, NULL }
-	  };
-	  etype = g_enum_register_static ("GOFileSaverLevel", values);
-  }
-  return etype;
+	static GType etype = 0;
+	if (etype == 0) {
+		static GEnumValue values[] = {
+			{ FILE_FL_NONE, (char*)"GO_FILE_SAVER_LEVEL_NONE", (char*)"none" },
+			{ FILE_FL_WRITE_ONLY, (char*)"GO_FILE_SAVER_LEVEL_WRITE_ONLY", (char*)"write_only" },
+			{ FILE_FL_NEW, (char*)"GO_FILE_SAVER_LEVEL_NEW", (char*)"new" },
+			{ FILE_FL_MANUAL, (char*)"GO_FILE_SAVER_LEVEL_MANUAL", (char*)"manual" },
+			{ FILE_FL_MANUAL_REMEMBER, (char*)"GO_FILE_SAVER_LEVEL_MANUAL_REMEMBER", (char*)"manual_remember" },
+			{ FILE_FL_AUTO, (char*)"GO_FILE_SAVER_LEVEL_AUTO", (char*)"auto" },
+			{ 0, NULL, NULL }
+		};
+		etype = g_enum_register_static ("GOFileSaverLevel", values);
+	}
+	return etype;
 }
 
 GType
 go_file_saver_scope_get_type (void)
 {
-  static GType etype = 0;
-  if (etype == 0) {
-	  static GEnumValue values[] = {
-		  { FILE_SAVE_WORKBOOK, (char*)"GO_FILE_SAVER_SCOPE_WORKBOOK", (char*)"workbook" },
-		  { FILE_SAVE_SHEET, (char*)"GO_FILE_SAVER_SCOPE_SHEET", (char*)"sheet" },
-		  { FILE_SAVE_RANGE, (char*)"GO_FILE_SAVER_SCOPE_RANGE", (char*)"range" },
-		  { 0, NULL, NULL }
-	  };
-	  etype = g_enum_register_static ("GOFileSaverScope", values);
-  }
-  return etype;
+	static GType etype = 0;
+	if (etype == 0) {
+		static GEnumValue values[] = {
+			{ FILE_SAVE_WORKBOOK, (char*)"GO_FILE_SAVER_SCOPE_WORKBOOK", (char*)"workbook" },
+			{ FILE_SAVE_SHEET, (char*)"GO_FILE_SAVER_SCOPE_SHEET", (char*)"sheet" },
+			{ FILE_SAVE_RANGE, (char*)"GO_FILE_SAVER_SCOPE_RANGE", (char*)"range" },
+			{ 0, NULL, NULL }
+		};
+		etype = g_enum_register_static ("GOFileSaverScope", values);
+	}
+	return etype;
 }
 
 /*-------------------------------------------------------------------------- */
@@ -729,7 +772,7 @@ typedef struct {
 } DefaultFileSaver;
 
 static GHashTable *file_opener_id_hash = NULL,
-                  *file_saver_id_hash = NULL;
+	*file_saver_id_hash = NULL;
 static GList *file_opener_list = NULL, *file_opener_priority_list = NULL;
 static GList *file_saver_list = NULL, *default_file_saver_list = NULL;
 
@@ -765,11 +808,11 @@ go_file_opener_register (GOFileOpener *fo, gint priority)
 	g_return_if_fail (priority >=0 && priority <= 100);
 
 	pos = go_list_index_custom (file_opener_priority_list,
-	                           GINT_TO_POINTER (priority),
-	                           cmp_int_less_than);
+				    GINT_TO_POINTER (priority),
+				    cmp_int_less_than);
 	file_opener_priority_list = g_list_insert (
-	                            file_opener_priority_list,
-	                            GINT_TO_POINTER (priority), pos);
+		file_opener_priority_list,
+		GINT_TO_POINTER (priority), pos);
 	file_opener_list = g_list_insert (file_opener_list, fo, pos);
 	g_object_ref (G_OBJECT (fo));
 
@@ -878,8 +921,8 @@ go_file_saver_register_as_default (GOFileSaver *fs, gint priority)
 	dfs->priority = priority;
 	dfs->saver = fs;
 	default_file_saver_list = g_list_insert_sorted (
-	                          default_file_saver_list, dfs,
-	                          default_file_saver_cmp_priority);
+		default_file_saver_list, dfs,
+		default_file_saver_cmp_priority);
 }
 
 /**
@@ -889,7 +932,7 @@ go_file_saver_register_as_default (GOFileSaver *fs, gint priority)
  *
  * Removes @fs saver from list of available file savers. Reference count
  * for the saver is decremented inside the function.
- */
+ **/
 void
 go_file_saver_unregister (GOFileSaver *fs)
 {
@@ -927,12 +970,12 @@ go_file_saver_unregister (GOFileSaver *fs)
 /**
  * go_file_saver_get_default:
  *
- * Returns file saver registered as default saver with the highest priority.
+ * Finds file saver registered as default saver with the highest priority.
  * Reference count for the saver is NOT incremented.
  *
- * Return value: GOFileSaver object or NULL if default saver is not
+ * Returns: GOFileSaver object or NULL if default saver is not
  *               available.
- */
+ **/
 GOFileSaver *
 go_file_saver_get_default (void)
 {
@@ -946,11 +989,8 @@ go_file_saver_get_default (void)
  * go_file_saver_for_mime_type:
  * @mime_type: A mime type
  *
- * Returns a file saver that claims to save files with given mime type.
- *
- * Return value: GOFileSaver object or NULL if no suitable file saver could
- *               be found.
- */
+ * Returns: A #GOFileSaver object associated with @mime_type, or %NULL
+ **/
 GOFileSaver *
 go_file_saver_for_mime_type (gchar const *mime_type)
 {
@@ -966,12 +1006,12 @@ go_file_saver_for_mime_type (gchar const *mime_type)
 
 /**
  * go_file_saver_for_file_name :
- * @file_name :
+ * @file_name : name
  *
  * Searches for file saver with given @filename, registered using
  * go_file_opener_register
  *
- * Return value: GOFileSaver object or NULL if opener cannot be found.
+ * Returns: GOFileSaver object or NULL if opener cannot be found.
  **/
 GOFileSaver *
 go_file_saver_for_file_name (char const *file_name)
@@ -987,13 +1027,13 @@ go_file_saver_for_file_name (char const *file_name)
 
 /**
  * go_file_opener_for_id:
- * @id          : File opener's ID
+ * @id : File opener's ID
  *
  * Searches for file opener with given @id, registered using
  * go_file_opener_register
  *
- * Return value: GOFileOpener object or NULL if opener cannot be found.
- */
+ * Returns: #GOFileOpener object or %NULL if opener cannot be found.
+ **/
 GOFileOpener *
 go_file_opener_for_id (gchar const *id)
 {
@@ -1006,13 +1046,13 @@ go_file_opener_for_id (gchar const *id)
 
 /**
  * go_file_saver_for_id:
- * @id          : File saver's ID
+ * @id : File saver's ID
  *
  * Searches for file saver with given @id, registered using
  * go_file_saver_register or register_file_opener_as_default.
  *
- * Return value: GOFileSaver object or NULL if saver cannot be found.
- */
+ * Returns: GOFileSaver object or NULL if saver cannot be found.
+ **/
 GOFileSaver *
 go_file_saver_for_id (gchar const *id)
 {
@@ -1026,10 +1066,7 @@ go_file_saver_for_id (gchar const *id)
 /**
  * go_get_file_savers:
  *
- * Returns the list of registered file savers (using go_file_saver_register or
- * go_file_saver_register_as_default).
- *
- * Return value: list of GOFileSaver objects, which you shouldn't modify.
+ * Returns: list of known# GOFileSaver types (do not modify list)
  */
 GList *
 go_get_file_savers (void)
@@ -1040,10 +1077,8 @@ go_get_file_savers (void)
 /**
  * go_get_file_openers:
  *
- * Returns the list of registered file openers (using go_file_opener_register).
- *
- * Return value: list of GOFileOpener objects, which you shouldn't modify.
- */
+ * Returns: list of known #GOFileOpener types (do not modify list)
+ **/
 GList *
 go_get_file_openers (void)
 {
