@@ -37,6 +37,7 @@
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-3d-box.h>
 #include <goffice/data/go-data-simple.h>
+#include <goffice/math/go-distribution.h>
 #include <goffice/math/go-math.h>
 #include <goffice/utils/go-format.h>
 #include <goffice/utils/go-font.h>
@@ -49,6 +50,16 @@
 
 #include <libintl.h>
 
+#ifdef G_OS_WIN32
+#define STRICT
+#include <windows.h>	// pickup HMODULE
+#undef STRICT
+#endif
+
+#ifdef GOFFICE_WITH_GMATHML
+#include <goffice/graph/gog-equation.h>
+#endif
+
 int goffice_graph_debug_level = 0;
 
 static char const *libgoffice_data_dir   = GOFFICE_DATADIR;
@@ -84,10 +95,17 @@ libgoffice_init ()
 
 #ifdef G_OS_WIN32
 	{
+	gchar *dir;
+
 #define S(s)	#s
-	gchar *dir = g_win32_get_package_installation_directory (NULL, (char *)
-		"libgoffice-" S(GO_VERSION_EPOCH) "-" S(GO_VERSION_MAJOR) ".dll");
+	char const *module_name = 
+		"libgoffice-" S(GO_VERSION_EPOCH) "-" S(GO_VERSION_MAJOR) ".dll";
 #undef S
+	wchar_t *wc_module_name = g_utf8_to_utf16 (module_name, -1, NULL, NULL, NULL);
+	HMODULE hmodule = GetModuleHandleW (wc_module_name);
+	g_free (wc_module_name);
+	dir = g_win32_get_package_installation_directory_of_module (hmodule);
+
 	libgoffice_data_dir = g_build_filename (dir,
 		"share", "goffice", GOFFICE_VERSION, NULL);
 	libgoffice_icon_dir = g_build_filename (dir,
@@ -102,6 +120,7 @@ libgoffice_init ()
 
 	bindtextdomain (GETTEXT_PACKAGE, libgoffice_locale_dir);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	g_type_init ();
 	go_conf_init ();
 	go_fonts_init ();
 	go_math_init ();
@@ -124,6 +143,9 @@ libgoffice_init ()
 	(void) GOG_LABEL_TYPE;
 	(void) GOG_GRID_TYPE;
 	(void) GOG_GRID_LINE_TYPE;
+#ifdef GOFFICE_WITH_GMATHML
+	(void) GOG_EQUATION_TYPE;
+#endif
 	(void) GOG_ERROR_BAR_TYPE;
 	(void) GOG_REG_EQN_TYPE;
 	(void) GOG_SERIES_LINES_TYPE;
@@ -133,6 +155,7 @@ libgoffice_init ()
 	gog_themes_init	();
 	go_number_format_init ();
 	go_currency_date_format_init ();
+	go_distributions_init ();
 	initialized = TRUE;
 }
 
