@@ -20,8 +20,11 @@
  */
 
 #include <goffice/graph/gog-chart-map.h>
+#include <goffice/math/go-cspline.h>
+#include <goffice/math/go-math.h>
+#include <goffice/math/go-rangefunc.h>
+#include <goffice/utils/go-bezier.h>
 #include <goffice/utils/go-line.h>
-#include <goffice/utils/go-math.h>
 #include <goffice/utils/go-path.h>
 #include <goffice/utils/go-geometry.h>
 
@@ -33,8 +36,9 @@ struct _GogChartMap {
 	gboolean		 is_valid;
 
 	void 	 (*map_2D_to_view) 	(GogChartMap *map, double x, double y, double *u, double *v);
+	double 	 (*map_2D_derivative_to_view) (GogChartMap *map, double deriv, double x, double y);
 	GOPath  *(*make_path)	   	(GogChartMap *map, double const *x, double const *y, int n_points,
-					 GOLineInterpolation interpolation);
+					 GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data);
 	GOPath  *(*make_close_path)	(GogChartMap *map, double const *x, double const *y, int n_points,
 					 GogSeriesFillType fill_type);
 };
@@ -181,10 +185,22 @@ xy_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
 	*v = gog_axis_map_to_view (map->axis_map[1], y);
 }
 
+static double
+xy_map_2D_derivative_to_view (GogChartMap *map, double deriv, double x, double y)
+{
+	double d;
+	d = gog_axis_map_derivative_to_view (map->axis_map[0], x);
+	if (isnan (d))
+		return go_nan;
+	deriv /= d;
+	d = gog_axis_map_derivative_to_view (map->axis_map[1], y);
+	return (isnan (d))? go_nan: deriv * d;
+}
+
 static GOPath *
-make_path_linear (GogChartMap *map, 
-		  double const *x, double const *y, 
-		  int n_points, gboolean is_polar)
+make_path_linear (GogChartMap *map,
+		  double const *x, double const *y,
+		  int n_points, gboolean is_polar, gboolean skip_invalid)
 {
 	GOPath *path;
 	int i, n_valid_points = 0;
@@ -208,10 +224,10 @@ make_path_linear (GogChartMap *map,
 			else if (!is_inverted && yy < yy_min)
 				yy = yy_min;
 		}
-		gog_chart_map_2D_to_view (map, 
+		gog_chart_map_2D_to_view (map,
 					  x != NULL ? x[i] : i + 1, yy,
 					  &xx, &yy);
-		if (go_finite (xx) 
+		if (go_finite (xx)
 		    && go_finite (yy)
 		    && fabs (xx) != DBL_MAX
 		    && fabs (yy) != DBL_MAX) {
@@ -219,9 +235,9 @@ make_path_linear (GogChartMap *map,
 
 			if (n_valid_points == 1)
 				go_path_move_to (path, xx, yy);
-			else 
+			else
 				go_path_line_to (path, xx, yy);
-		} else 
+		} else if (!skip_invalid)
 			n_valid_points = 0;
 	}
 
@@ -229,17 +245,18 @@ make_path_linear (GogChartMap *map,
 }
 
 static GOPath *
-make_path_spline (GogChartMap *map, 
+make_path_spline (GogChartMap *map,
 		  double const *x, double const *y, int n_points,
-		  gboolean is_polar)
+		  gboolean is_polar, gboolean closed, gboolean skip_invalid)
 {
 	GOPath *path;
-	int i, ii = 0, ii_prev = 0, ii_prev_prev, n_valid_points = 0;
-	double angles[3], lengths[3], thetas[3];
-	double uu[3], vv[3];
+	int i, n_valid_points = 0;
+	double *uu, *vv, *tt;
 	double u, v;
 	double yy, yy_min, yy_max;
 	gboolean is_inverted;
+	//struct GOCSpline *splinex, *spliney;
+	struct GOBezierSpline *spline;
 
 	path = go_path_new ();
 	if (n_points < 1)
@@ -248,8 +265,11 @@ make_path_spline (GogChartMap *map,
 	gog_axis_map_get_bounds (map->axis_map[1], &yy_min, &yy_max);
 	is_inverted = gog_axis_map_is_inverted (map->axis_map[1]);
 
+	uu = g_new (double, n_points);
+	vv = g_new (double, n_points);
+	tt = g_new (double, n_points);
 	n_valid_points = 0;
-	for (i = 0; i <= n_points; i++) {
+	for (i = 0; i < n_points; i++) {
 		yy = y != NULL ? y[i] : i + 1;
 		if (is_polar) {
 			if (is_inverted && yy > yy_max)
@@ -257,79 +277,146 @@ make_path_spline (GogChartMap *map,
 			else if (!is_inverted && yy < yy_min)
 				yy = yy_min;
 		}
-		gog_chart_map_2D_to_view (map, 
-					  x != NULL ? x[i] : i + 1, yy, 
+		gog_chart_map_2D_to_view (map,
+					  x != NULL ? x[i] : i + 1, yy,
 					  &u, &v);
-		if (go_finite (u) 
+		if (go_finite (u)
 		    && go_finite (v)
 		    && fabs (u) != DBL_MAX
-		    && fabs (v) != DBL_MAX
-		    && i != n_points) {
+		    && fabs (v) != DBL_MAX) {
+			uu[n_valid_points] = u;
+			vv[n_valid_points] = v;
+			tt[n_valid_points] = n_valid_points;
 			n_valid_points++;
-
-			ii = i % 3;
-			ii_prev = (i - 1) % 3;
-			ii_prev_prev = (i - 2) % 3;
-
-			uu[ii] = u;
-			vv[ii] = v;
-
-			if (n_valid_points == 1) {
-				go_path_move_to (path, u, v);
-				angles[ii] = 0;
-				lengths[ii] = 0;
-			}
-
-			if (n_valid_points > 1) {
-				go_geometry_cartesian_to_polar (u - uu[ii_prev],
-								v - vv[ii_prev],
-								&lengths[ii],
-								&angles[ii]);
-				lengths[ii] /= 4.0;
-				if (fabs (angles[ii] - angles[ii_prev]) > M_PI)
-					angles[ii] -= (angles[ii] > angles[ii_prev]) ?
-						2. * M_PI :
-					       -2. * M_PI;
-				thetas[ii] = (angles[ii_prev] * lengths[ii] 
-					      + angles[ii] * lengths[ii_prev]) 
-					/ (lengths[ii] + lengths[ii_prev]);
-				if (!go_finite (thetas[ii]))
-					thetas[ii] = 0;
-			}
-
-			if (n_valid_points == 3)
-				go_path_curve_to (path, 
-						  uu[ii_prev_prev],
-						  vv[ii_prev_prev],
-						  uu[ii_prev]      - lengths[ii_prev] * cos (thetas[ii]),
-						  vv[ii_prev]      - lengths[ii_prev] * sin (thetas[ii]),
-						  uu[ii_prev], vv[ii_prev]);
-			else if (n_valid_points > 3)
-				go_path_curve_to (path, 
-						  uu[ii_prev_prev] + lengths[ii_prev] * cos (thetas[ii_prev]),
-						  vv[ii_prev_prev] + lengths[ii_prev] * sin (thetas[ii_prev]),
-						  uu[ii_prev]      - lengths[ii_prev] * cos (thetas[ii]),
-						  vv[ii_prev]      - lengths[ii_prev] * sin (thetas[ii]),
-						  uu[ii_prev], vv[ii_prev]);
 		} else {
-			if (n_valid_points == 2) 
-				go_path_line_to (path, uu[ii], vv[ii]);
-			else if (n_valid_points > 2) 
-				go_path_curve_to (path, 
-						  uu[ii_prev] + lengths[ii] * cos (thetas[ii]),
-						  vv[ii_prev] + lengths[ii] * sin (thetas[ii]),
-						  uu[ii], vv[ii], 
-						  uu[ii], vv[ii]);
+			if (closed || skip_invalid)
+				continue;
+			if (n_valid_points == 2) {
+				go_path_move_to (path, uu[0], vv[0]);
+				go_path_line_to (path, uu[1], vv[1]);
+			} else if (n_valid_points > 2) {
+				/* evaluate the spline */
+				spline = go_bezier_spline_init (uu, vv, n_valid_points, closed);
+				path = go_bezier_spline_to_path (spline);
+				go_bezier_spline_destroy (spline);
+			}
 			n_valid_points = 0;
 		}
 	}
+	if (n_valid_points == 2) {
+		go_path_move_to (path, uu[0], vv[0]);
+		go_path_line_to (path, uu[1], vv[1]);
+	} else if (n_valid_points > 2) {
+		/* evaluate the spline */
+		spline = go_bezier_spline_init (uu, vv, n_valid_points, closed);
+		path = go_bezier_spline_to_path (spline);
+		go_bezier_spline_destroy (spline);
+	}
+
+	g_free (uu);
+	g_free (vv);
+	g_free (tt);
+	return path;
+}
+
+static GOPath *
+make_path_cspline (GogChartMap *map,
+		  double const *x, double const *y, int n_points,
+		  gboolean is_polar, GOCSplineType type, gboolean skip_invalid,
+		  gpointer data)
+{
+	GOPath *path;
+	int i, n_valid_points = 0;
+	double *uu, *vv, u, v;
+	double p0 = 0., p1 = 0.; /* clamped derivatives */
+	struct GOCSpline *spline;
+
+	path = go_path_new ();
+	if (n_points < 1 || !go_range_increasing (x, n_points))
+		return path;
+
+	uu = g_new (double, n_points);
+	vv = g_new (double, n_points);
+	n_valid_points = 0;
+
+	for (i = 0; i < n_points; i++) {
+		gog_chart_map_2D_to_view (map,
+					  x != NULL ? x[i] : i + 1, y != NULL ? y[i] : i + 1,
+					  &u, &v);
+		if (go_finite (u)
+		    && go_finite (v)
+		    && fabs (u) != DBL_MAX
+		    && fabs (v) != DBL_MAX) {
+			uu[n_valid_points] = u;
+			vv[n_valid_points] = v;
+			n_valid_points++;
+		} else {
+			if (skip_invalid || type == GO_CSPLINE_CLAMPED)
+				continue;
+			if (n_valid_points == 2)
+				go_path_line_to (path, uu[1], vv[1]);
+			else if (n_valid_points > 2) {
+				/* evaluate the spline */
+				spline = go_cspline_init (uu, vv, n_valid_points, type, p0, p1);
+				if (spline) {
+					double x0, x1, x2, x3, y0, y1, y2, y3;
+					x0 = uu[0];
+					y0 = vv[0];
+					go_path_move_to (path, x0, y0);
+					for (i = 1; i < spline->n; i++) {
+						x3 = uu[i];
+						y3 = vv[i];
+						u = x3 - x0;
+						x1 = (2. * x0 + x3) / 3.;
+						x2 = (x0 + 2. * x3) / 3.;
+						y1 = y0 + spline->c[i-1] / 3. * u;
+						y2 = y0 + (2. * spline->c[i-1] + spline->b[i-1] * u) / 3. * u;
+						go_path_curve_to (path, x1, y1, x2, y2, x3, y3);
+						x0 = x3;
+						y0 = y3;
+					}
+					go_cspline_destroy (spline);
+				}
+			}
+		}
+	}
+	if (n_valid_points == 2) {
+		go_path_move_to (path, uu[0], vv[0]);
+		go_path_line_to (path, uu[1], vv[1]);
+	} else if (n_valid_points > 2) {
+		if (type == GO_CSPLINE_CLAMPED && data != NULL) {
+			p0 = gog_chart_map_2D_derivative_to_view (map, ((double*) data)[0], uu[0], vv[0]);
+			p1 = gog_chart_map_2D_derivative_to_view (map, ((double*) data)[1],
+								  uu[n_valid_points - 1], vv[n_valid_points - 1]);
+		}
+		spline = go_cspline_init (uu, vv, n_valid_points, type, p0, p1);
+		if (spline) {
+			double x0, x1, x2, x3, y0, y1, y2, y3;
+			x0 = uu[0];
+			y0 = vv[0];
+			go_path_move_to (path, x0, y0);
+			for (i = 1; i < spline->n; i++) {
+				x3 = uu[i];
+				y3 = vv[i];
+				u = x3 - x0;
+				x1 = (2. * x0 + x3) / 3.;
+				x2 = (x0 + 2. * x3) / 3.;
+				y1 = y0 + spline->c[i-1] / 3. * u;
+				y2 = y0 + (2. * spline->c[i-1] + spline->b[i-1] * u) / 3. * u;
+				go_path_curve_to (path, x1, y1, x2, y2, x3, y3);
+				x0 = x3;
+				y0 = y3;
+			}
+			go_cspline_destroy (spline);
+		}
+	}
 
 	return path;
 }
 
 static GOPath *
 xy_make_path_step (GogChartMap *map, double const *x, double const *y, int n_points,
-		   GOLineInterpolation interpolation)
+		   GOLineInterpolation interpolation, gboolean skip_invalid)
 {
 	GOPath *path;
 	int i, n_valid_points = 0;
@@ -375,7 +462,7 @@ xy_make_path_step (GogChartMap *map, double const *x, double const *y, int n_poi
 			go_path_line_to (path, xx, yy);
 			last_xx = xx;
 			last_yy = yy;
-		} else 
+		} else if (!skip_invalid)
 			n_valid_points = 0;
 	}
 
@@ -386,22 +473,37 @@ xy_make_path_step (GogChartMap *map, double const *x, double const *y, int n_poi
 
 static GOPath *
 xy_make_path (GogChartMap *map, double const *x, double const *y, 
-	      int n_points, GOLineInterpolation interpolation)
+	      int n_points, GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data)
 {
 	GOPath *path = NULL;
 
 	switch (interpolation) {
 		case GO_LINE_INTERPOLATION_LINEAR:
-			path = make_path_linear (map, x, y, n_points, FALSE);
+			path = make_path_linear (map, x, y, n_points, FALSE, skip_invalid);
 			break;
 		case GO_LINE_INTERPOLATION_SPLINE:
-			path = make_path_spline (map, x, y, n_points, FALSE);
+			path = make_path_spline (map, x, y, n_points, FALSE, FALSE, skip_invalid);
+			break;
+		case GO_LINE_INTERPOLATION_CLOSED_SPLINE:
+			path = make_path_spline (map, x, y, n_points, TRUE, TRUE, skip_invalid);
+			break;
+		case GO_LINE_INTERPOLATION_CUBIC_SPLINE:
+			path = make_path_cspline (map, x, y, n_points, TRUE, GO_CSPLINE_NATURAL, skip_invalid, data);
+			break;
+		case GO_LINE_INTERPOLATION_PARABOLIC_CUBIC_SPLINE:
+			path = make_path_cspline (map, x, y, n_points, TRUE, GO_CSPLINE_PARABOLIC, skip_invalid, data);
+			break;
+		case GO_LINE_INTERPOLATION_CUBIC_CUBIC_SPLINE:
+			path = make_path_cspline (map, x, y, n_points, TRUE, GO_CSPLINE_CUBIC, skip_invalid, data);
+			break;
+		case GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE:
+			path = make_path_cspline (map, x, y, n_points, TRUE, GO_CSPLINE_CLAMPED, skip_invalid, data);
 			break;
 		case GO_LINE_INTERPOLATION_STEP_START:
 		case GO_LINE_INTERPOLATION_STEP_END:
 		case GO_LINE_INTERPOLATION_STEP_CENTER_X:
 		case GO_LINE_INTERPOLATION_STEP_CENTER_Y:
-			path = xy_make_path_step (map, x, y, n_points, interpolation);
+			path = xy_make_path_step (map, x, y, n_points, interpolation, skip_invalid);
 			break;
 		default:
 			g_assert_not_reached ();
@@ -443,8 +545,8 @@ xy_make_close_path (GogChartMap *map, double const *x, double const *y,
 			close_path = go_path_new ();
 			gog_axis_map_get_extents (y_map, &position, NULL);
 			position = gog_axis_map_to_view (y_map, position);
-			go_path_move_to (close_path, gog_axis_map_to_view (x_map, x_start), position); 
-			go_path_line_to (close_path, gog_axis_map_to_view (x_map, x_end), position); 
+			go_path_move_to (close_path, gog_axis_map_to_view (x_map, x_start), position);
+			go_path_line_to (close_path, gog_axis_map_to_view (x_map, x_end), position);
 			break;
 		case GOG_SERIES_FILL_TYPE_LEFT:
 			close_path = go_path_new ();
@@ -457,8 +559,8 @@ xy_make_close_path (GogChartMap *map, double const *x, double const *y,
 			close_path = go_path_new ();
 			gog_axis_map_get_extents (y_map, NULL, &position);
 			position = gog_axis_map_to_view (y_map, position);
-			go_path_move_to (close_path, gog_axis_map_to_view (x_map, x_start), position); 
-			go_path_line_to (close_path, gog_axis_map_to_view (x_map, x_end), position); 
+			go_path_move_to (close_path, gog_axis_map_to_view (x_map, x_start), position);
+			go_path_line_to (close_path, gog_axis_map_to_view (x_map, x_end), position);
 			break;
 		case GOG_SERIES_FILL_TYPE_RIGHT:
 			close_path = go_path_new ();
@@ -482,14 +584,14 @@ polar_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v
 	GogChartMapPolarData *data = (GogChartMapPolarData *) map->data;
 	double r = gog_axis_map_to_view (map->axis_map[1], y);
 	double t = gog_axis_map_to_view (map->axis_map[0], x);
-	
+
 	*u = data->cx + r * data->rx * cos (t);
-	*v = data->cy + r * data->ry * sin (t);		
+	*v = data->cy + r * data->ry * sin (t);
 }
 
 static GOPath *
 polar_make_path_step (GogChartMap *map, double const *x, double const *y, int n_points,
-		      GOLineInterpolation interpolation)
+		      GOLineInterpolation interpolation, gboolean skip_invalid)
 {
 	GogChartMapPolarData *polar_parms;
 	GOPath *path;
@@ -522,8 +624,8 @@ polar_make_path_step (GogChartMap *map, double const *x, double const *y, int n_
 		theta = gog_axis_map_to_view (map->axis_map[0], x != NULL ? x[i] : i + 1);
 		rho = gog_axis_map_to_view (map->axis_map[1], rho);
 		xx = cx + rho * rx * cos (theta);
-		yy = cy + rho * ry * sin (theta);		
-		if (go_finite (xx) 
+		yy = cy + rho * ry * sin (theta);
+		if (go_finite (xx)
 		    && go_finite (yy)
 		    && fabs (xx) != DBL_MAX
 		    && fabs (yy) != DBL_MAX) {
@@ -566,7 +668,7 @@ polar_make_path_step (GogChartMap *map, double const *x, double const *y, int n_
 			last_yy = yy;
 			last_theta = theta;
 			last_rho = rho;
-		} else
+		} else if (!skip_invalid)
 			n_valid_points = 0;
 	}
 
@@ -574,23 +676,32 @@ polar_make_path_step (GogChartMap *map, double const *x, double const *y, int n_
 }
 
 static GOPath *
-polar_make_path (GogChartMap *map, double const *x, double const *y, 
-		 int n_points, GOLineInterpolation interpolation)
+polar_make_path (GogChartMap *map, double const *x, double const *y,
+		 int n_points, GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data)
 {
 	GOPath *path = NULL;
 
 	switch (interpolation) {
 		case GO_LINE_INTERPOLATION_LINEAR:
-			path = make_path_linear (map, x, y, n_points, TRUE);
+			path = make_path_linear (map, x, y, n_points, TRUE, skip_invalid);
 			break;
 		case GO_LINE_INTERPOLATION_SPLINE:
-			path = make_path_spline (map, x, y, n_points, TRUE);
+			path = make_path_spline (map, x, y, n_points, TRUE, FALSE, skip_invalid);
+			break;
+		case GO_LINE_INTERPOLATION_CLOSED_SPLINE:
+			path = make_path_spline (map, x, y, n_points, TRUE, TRUE, skip_invalid);
+			break;
+		case GO_LINE_INTERPOLATION_CUBIC_SPLINE:
+		case GO_LINE_INTERPOLATION_PARABOLIC_CUBIC_SPLINE:
+		case GO_LINE_INTERPOLATION_CUBIC_CUBIC_SPLINE:
+		case GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE:
+			/* Not supported in polar plots */
 			break;
 		case GO_LINE_INTERPOLATION_STEP_START:
 		case GO_LINE_INTERPOLATION_STEP_END:
 		case GO_LINE_INTERPOLATION_STEP_CENTER_X:
 		case GO_LINE_INTERPOLATION_STEP_CENTER_Y:
-			path = polar_make_path_step (map, x, y, n_points, interpolation);
+			path = polar_make_path_step (map, x, y, n_points, interpolation, skip_invalid);
 			break;
 		default:
 			g_assert_not_reached ();
@@ -600,7 +711,7 @@ polar_make_path (GogChartMap *map, double const *x, double const *y,
 }
 
 static GOPath *
-polar_make_close_path (GogChartMap *map, double const *x, double const *y, 
+polar_make_close_path (GogChartMap *map, double const *x, double const *y,
 		 int n_points, GogSeriesFillType fill_type)
 {
 	GogChartMapPolarData *parms = map->data;
@@ -656,7 +767,7 @@ GogChartMapPolarData *
 gog_chart_map_get_polar_parms (GogChartMap *map)
 {
 	return (GogChartMapPolarData *) map->data;
-}	
+}
 
 /**
  * gog_chart_map_new:
@@ -674,7 +785,7 @@ gog_chart_map_get_polar_parms (GogChartMap *map)
  **/
 
 GogChartMap *
-gog_chart_map_new (GogChart *chart, GogViewAllocation const *area, 
+gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 		   GogAxis *axis0, GogAxis *axis1, GogAxis *axis2,
 		   gboolean fill_area)
 {
@@ -682,6 +793,9 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 	GogAxisSet axis_set;
 
 	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
+	axis_set = gog_chart_get_axis_set (chart);
+	g_return_val_if_fail (axis_set != GOG_AXIS_SET_UNKNOWN &&
+			      axis_set != GOG_AXIS_SET_NONE, NULL);
 
 	map = g_new (GogChartMap, 1);
 
@@ -690,20 +804,20 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 	map->area = *area;
 	map->data = NULL;
 	map->is_valid = FALSE;
+	map->axis_map[0] = map->axis_map[1] = map->axis_map[2] = NULL;
 
-	axis_set = gog_chart_get_axis_set (chart);
 	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
 		case GOG_AXIS_SET_X:
 			{
 				XMapData *data = g_new (XMapData, 1);
 
 				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = map->axis_map[2] = NULL;
 
 				data->b = area->y + area->h;
 				data->a = - area->h;
 
 				map->map_2D_to_view = x_map_2D_to_view;
+				map->map_2D_derivative_to_view = NULL;
 				map->make_path = NULL;
 				map->make_close_path = NULL;
 				map->data = data;
@@ -716,12 +830,12 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 		case GOG_AXIS_SET_XY_COLOR:
 			{
 				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
-				map->axis_map[1] = gog_axis_map_new (axis1, map->area.y + map->area.h, 
+				map->axis_map[1] = gog_axis_map_new (axis1, map->area.y + map->area.h,
 								     -map->area.h);
-				map->axis_map[2] = NULL;
 
 				map->data = NULL;
 				map->map_2D_to_view = xy_map_2D_to_view;
+				map->map_2D_derivative_to_view = xy_map_2D_derivative_to_view;
 				map->make_path = xy_make_path;
 				map->make_close_path = xy_make_close_path;
 
@@ -735,7 +849,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 				double minimum, maximum;
 				double z_rotation = gog_axis_get_circular_rotation (axis0) * M_PI / 180.0;
 				double perimeter;
-	
+
 				map->axis_map[0] = gog_axis_map_new (axis0, 0.0, 1.0);
 				gog_axis_map_get_bounds (map->axis_map[0], &minimum, &maximum);
 				if (gog_axis_is_discrete (axis0)) {
@@ -743,7 +857,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 					data->th1 = go_rint (maximum);
 					calc_polygon_parameters (area, data, fill_area);
 					gog_axis_map_free (map->axis_map[0]);
-					map->axis_map[0] = gog_axis_map_new (axis0, 
+					map->axis_map[0] = gog_axis_map_new (axis0,
 						- M_PI / 2.0 + z_rotation,
 						2.0 * M_PI * (maximum - minimum) / (maximum - minimum + 1));
 				} else {
@@ -758,10 +872,10 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 									     minimum - maximum);
 				}
 				map->axis_map[1] = gog_axis_map_new (axis1, 0.0, 1.0);
-				map->axis_map[2] = NULL;
 
 				map->data = data;
 				map->map_2D_to_view = polar_map_2D_to_view;
+				map->map_2D_derivative_to_view = NULL;
 				map->make_path = polar_make_path;
 				map->make_close_path = polar_make_close_path;
 
@@ -769,14 +883,10 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 					gog_axis_map_is_valid (map->axis_map[1]);
 				break;
 			}
-		case GOG_AXIS_SET_XYZ:
-		case GOG_AXIS_SET_ALL:
-		case GOG_AXIS_SET_NONE:
-		case GOG_AXIS_SET_UNKNOWN:
 		default:
-			g_warning ("[Chart::map_new] not implemented for this axis set (%i)",
-				   axis_set);
+			g_warning ("[GogChartMap::new] unimplemented for axis set %d", axis_set);
 			map->map_2D_to_view = null_map_2D;
+			map->map_2D_derivative_to_view = NULL;
 			break;
 	}
 
@@ -800,6 +910,26 @@ gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, doubl
 	(map->map_2D_to_view) (map, x, y, u, v);
 }
 
+
+/**
+ * gog_chart_map_2D_derivative_to_view:
+ * @map: a #GogChartMap
+ * @deriv: the slope in data space
+ * @x: data x value
+ * @y: data y value
+ *
+ * Converts a 2D slope from data space to canvas space. It is only implemented
+ for xy maps.
+ * Returns: the slope in canvas space or go_nan.
+ **/
+
+double
+gog_chart_map_2D_derivative_to_view (GogChartMap *map, double deriv, double x, double y)
+{
+	return (map->map_2D_derivative_to_view)?
+		map->map_2D_derivative_to_view (map, deriv, x, y): go_nan;
+}
+
 /**
  * gog_chart_map_get_axis_map:
  * @map: a #GogChartMap
@@ -871,17 +1001,17 @@ gog_chart_map_free (GogChartMap *map)
  * @y: y data
  * @n_points: number of points
  * @interpolation: interpolation type
+ * @skip_invalid: whether to ignore invalid data or interrupt the interpolation
  *
- * Creates a new GOPath using @x and @y data, each valid point being connected with respect to @interpolation.
+ * Returns: a new GOPath using @x and @y data, each valid point being connected with respect to @interpolation.
  **/
-
 GOPath *
-gog_chart_map_make_path (GogChartMap *map, double const *x, double const *y, 
-			 int n_points, 
-			 GOLineInterpolation interpolation)
+gog_chart_map_make_path (GogChartMap *map, double const *x, double const *y,
+			 int n_points,
+			 GOLineInterpolation interpolation, gboolean skip_invalid, gpointer data)
 {
 	if (map->make_path != NULL)
-		return (map->make_path) (map, x, y, n_points, interpolation);
+		return (map->make_path) (map, x, y, n_points, interpolation, skip_invalid, data);
 
 	return NULL;
 }
@@ -892,14 +1022,13 @@ gog_chart_map_make_path (GogChartMap *map, double const *x, double const *y,
  * @x: x data
  * @y: y data
  * @n_points: number of points
- * @interpolation: interpolation type
+ * @fill_type: fill type
  *
- * Creates a new GOPath using @x and @y data, each valid point being connected with respect to @interpolation.
+ * Returns: a new GOPath using @x and @y data, with respect to @fill_type.
  **/
-
 GOPath *
-gog_chart_map_make_close_path (GogChartMap *map, double const *x, double const *y, 
-			       int n_points, 
+gog_chart_map_make_close_path (GogChartMap *map, double const *x, double const *y,
+			       int n_points,
 			       GogSeriesFillType fill_type)
 {
 	if (map->make_close_path != NULL)
