@@ -37,9 +37,7 @@
 #include <stdlib.h>
 
 #ifdef GOFFICE_WITH_GTK
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkliststore.h>
+#include <gtk/gtk.h>
 #endif
 
 #define GOG_GRAPH_DEFAULT_WIDTH 	GO_CM_TO_PT (12.0)
@@ -92,7 +90,7 @@ gog_graph_set_property (GObject *obj, guint param_id,
 		GObject *object = g_value_get_object (value);
 		if ((GODoc *) object == graph->doc)
 			break;
-		if (IS_GO_DOC (obj)) {
+		if (IS_GO_DOC (object)) {
 			graph->doc = (GODoc *) object;
 			gog_object_document_changed (GOG_OBJECT (graph), graph->doc);
 		}
@@ -280,12 +278,12 @@ gog_graph_class_init (GogGraphClass *klass)
 	GogObjectClass *gog_klass = (GogObjectClass *) klass;
 
 	static GogObjectRole const roles[] = {
-		{ N_("Chart"), "GogChart",	0,
+		{ N_("Chart"), "GogChart",	1,
 		  GOG_POSITION_SPECIAL|GOG_POSITION_ANY_MANUAL, 
 		  GOG_POSITION_SPECIAL, 
 		  GOG_OBJECT_NAME_BY_ROLE,
 		  NULL, NULL, NULL, role_chart_post_add, role_chart_pre_remove, NULL },
-		{ N_("Title"), "GogLabel",	1,
+		{ N_("Title"), "GogLabel",	0,
 		  GOG_POSITION_COMPASS|GOG_POSITION_ANY_MANUAL, 
 		  GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER, 
 		  GOG_OBJECT_NAME_BY_ROLE,
@@ -912,7 +910,7 @@ update_action (GogGraphView *view, GogTool *tool, double x, double y)
 }
 
 #ifdef GOFFICE_WITH_GTK
-#include <gdk/gdkcursor.h>
+#include <gdk/gdk.h>
 
 /**
  * gog_graph_view_handle_event:
