@@ -23,7 +23,7 @@
 
 #include <goffice/graph/goffice-graph.h>
 #include <goffice/app/goffice-app.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -32,8 +32,11 @@ G_BEGIN_DECLS
 /* typedef void (*GogGuruRegister) (GogGraph *graph, gpointer user); */
 
 GtkWidget *gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
-		     GOCmdContext *cc, GtkWindow *toplevel,
-		     GClosure *closure);
+		     GOCmdContext *cc, GClosure *closure);
+
+GtkWidget *gog_guru_get_help_button (GtkWidget *guru);
+
+void	   gog_guru_add_custom_widget (GtkWidget *guru, GtkWidget *custom);
 
 G_END_DECLS
 
