@@ -36,14 +36,7 @@
 #include <glib/gi18n-lib.h>
 
 #ifdef GOFFICE_WITH_GTK
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtktable.h>
+#include <gtk/gtk.h>
 #endif
 
 #include <gsf/gsf-impl-utils.h>
@@ -87,13 +80,6 @@ role_series_can_add (GogObject const *parent)
 	return g_slist_length (plot->series) < plot->desc.num_series_max;
 }
 
-static gboolean
-role_series_can_remove (GogObject const *child)
-{
-	GogPlot const *plot = GOG_PLOT (child->parent);
-	return g_slist_length (plot->series) > plot->desc.num_series_min;
-}
-
 static GogObject *
 role_series_allocate (GogObject *plot)
 {
@@ -509,7 +495,7 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 	static GogObjectRole const roles[] = {
 		{ N_("Series"), "GogSeries",	0,
 		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
-		  role_series_can_add, role_series_can_remove,
+		  role_series_can_add, NULL,
 		  role_series_allocate,
 		  role_series_post_add, role_series_pre_remove, NULL },
 	};
@@ -679,7 +665,7 @@ gog_plot_update_cardinality (GogPlot *plot, int index_num)
 			gog_series_set_index (series, i++, FALSE);
 			if (!gog_series_has_legend (series))
 				no_legend++;
-		}
+		}			
 	}
 
 	plot->full_cardinality =
@@ -1062,6 +1048,15 @@ gog_plot_guru_helper (GogPlot *plot)
 	g_strfreev (hints);
 }
 
+void gog_plot_clear_series (GogPlot *plot)
+{
+	while (plot->series) {
+		GogObject *gobj = GOG_OBJECT (plot->series->data);
+		gog_object_clear_parent (gobj);
+		g_object_unref (gobj);
+	}
+}
+
 /*****************************************************************************/
 
 #ifdef GOFFICE_WITH_GTK
