@@ -27,21 +27,13 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-error-bar.h>
 #include <goffice/data/go-data.h>
+#include <goffice/utils/go-persist.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhseparator.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkcombobox.h>
 #endif
 
 #include <string.h>
@@ -229,7 +221,7 @@ gog_series_element_class_init (GogSeriesElementClass *klass)
 			_("Index"),
 			_("Index of the corresponding data element"),
 			0, G_MAXINT, 0,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT | GOG_PARAM_FORCE_SAVE));
 }
 
 GSF_CLASS (GogSeriesElement, gog_series_element,
@@ -264,6 +256,7 @@ enum {
 	SERIES_PROP_0,
 	SERIES_PROP_HAS_LEGEND,
 	SERIES_PROP_INTERPOLATION,
+	SERIES_PROP_INTERPOLATION_SKIP_INVALID,
 	SERIES_PROP_FILL_TYPE
 };
 
@@ -347,6 +340,9 @@ gog_series_set_property (GObject *obj, guint param_id,
 	case SERIES_PROP_INTERPOLATION:
 		series->interpolation = go_line_interpolation_from_str (g_value_get_string (value));
 		break;
+	case SERIES_PROP_INTERPOLATION_SKIP_INVALID:
+		series->interpolation_skip_invalid = g_value_get_boolean (value);
+		break;
 	case SERIES_PROP_FILL_TYPE:
 		name = g_value_get_string (value);
 		for (i = 0; i < G_N_ELEMENTS (_fill_type_infos); i++)
@@ -374,6 +370,9 @@ gog_series_get_property (GObject *obj, guint param_id,
 	case SERIES_PROP_INTERPOLATION:
 		g_value_set_string (value, go_line_interpolation_as_str (series->interpolation));
 		break;
+	case SERIES_PROP_INTERPOLATION_SKIP_INVALID:
+		g_value_set_boolean (value, series->interpolation_skip_invalid);
+		break;
 	case SERIES_PROP_FILL_TYPE:
 		g_value_set_string (value, _fill_type_infos[series->fill_type].name);
 		break;
@@ -416,7 +415,25 @@ cb_show_in_legend (GtkToggleButton *b, GObject *series)
 static void
 cb_line_interpolation_changed (GtkComboBox *box, GogSeries *series)
 {
+	GladeXML *gui = g_object_get_data (G_OBJECT (box), "gui");
+	GtkWidget *widget = GTK_WIDGET (g_object_get_data (G_OBJECT(box), "skip-button"));
+	GtkWidget *table = glade_xml_get_widget (gui, "clamps-table");
 	series->interpolation = gtk_combo_box_get_active (box);
+	gtk_widget_set_sensitive (widget, !go_line_interpolation_auto_skip (series->interpolation));
+	widget = GTK_WIDGET (g_object_get_data (G_OBJECT(box), "fill-type"));
+	if (series->interpolation == GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE)
+		gtk_widget_show (table);
+	else
+		gtk_widget_hide (table);
+	if (widget)
+		gtk_widget_set_sensitive (widget, !go_line_interpolation_auto_skip (series->interpolation));
+	gog_object_emit_changed (GOG_OBJECT (series), FALSE);
+}
+
+static void
+cb_line_interpolation_skip_changed (GtkToggleButton *btn, GogSeries *series)
+{
+	series->interpolation_skip_invalid = gtk_toggle_button_get_active (btn);
 	gog_object_emit_changed (GOG_OBJECT (series), FALSE);
 }
 
@@ -443,6 +460,7 @@ gog_series_populate_editor (GogObject *gobj,
 	GogDataset *set = GOG_DATASET (gobj);
 	GogSeriesDesc const *desc;
 	GogDataType data_type;
+	GtkComboBox *combo = NULL;
 
 	g_return_if_fail (series->plot != NULL);
 
@@ -514,13 +532,49 @@ gog_series_populate_editor (GogObject *gobj,
 
 		gui = go_libglade_new ("gog-series-prefs.glade", "interpolation_prefs", GETTEXT_PACKAGE, cc);
 		if (gui != NULL) {
+			int i;
+			GogAxisSet set = gog_plot_axis_set_pref (gog_series_get_plot (series));
+			GogDataset *clamp_set = gog_series_get_interpolation_params (series);
 			widget = glade_xml_get_widget (gui, "interpolation_prefs");
 			gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
-			widget = glade_xml_get_widget (gui, "interpolation_combo");
-			gtk_combo_box_set_active (GTK_COMBO_BOX (widget), series->interpolation);
-			g_signal_connect (widget, "changed",
+			widget = glade_xml_get_widget (gui, "interpolation-table");
+			/* create an interpolation type combo and populate it */
+			combo = GTK_COMBO_BOX (gtk_combo_box_new_text ());
+			if (set & 1 << GOG_AXIS_RADIAL)
+				for (i = 0; i < GO_LINE_INTERPOLATION_MAX; i++) {
+					if (go_line_interpolation_supports_radial (i))
+						gtk_combo_box_append_text (combo, _(go_line_interpolation_as_label (i)));
+				}
+			else
+				for (i = 0; i < GO_LINE_INTERPOLATION_MAX; i++)
+					gtk_combo_box_append_text (combo, _(go_line_interpolation_as_label (i)));
+			gtk_combo_box_set_active (combo, series->interpolation);
+			g_signal_connect (combo, "changed",
 					  G_CALLBACK (cb_line_interpolation_changed), series);
-			g_object_set_data_full (G_OBJECT (widget), "gui", gui,
+			gtk_table_attach (GTK_TABLE (widget), GTK_WIDGET (combo), 1, 2,
+					  0, 1, (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
+					  (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), 0, 0);
+			gtk_widget_show_all (widget);
+			widget = glade_xml_get_widget (gui, "interpolation-skip-invalid");
+			g_object_set_data (G_OBJECT (combo), "skip-button", widget); 
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), series->interpolation_skip_invalid);
+			if (go_line_interpolation_auto_skip (series->interpolation))
+				gtk_widget_set_sensitive (widget, FALSE);
+			g_signal_connect (widget, "toggled",
+					  G_CALLBACK (cb_line_interpolation_skip_changed), series);
+			if (clamp_set) {
+				GtkWidget *w;
+				widget = glade_xml_get_widget (gui, "clamps-table");
+				w = GTK_WIDGET (gog_data_allocator_editor (dalloc, clamp_set, 0, GOG_DATA_SCALAR));
+				gtk_widget_show (w);
+				gtk_table_attach (GTK_TABLE (widget), w, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				w = GTK_WIDGET (gog_data_allocator_editor (dalloc, clamp_set, 1, GOG_DATA_SCALAR));
+				gtk_widget_show (w);
+				gtk_table_attach (GTK_TABLE (widget), w, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+				if (series->interpolation != GO_LINE_INTERPOLATION_CLAMPED_CUBIC_SPLINE)
+					gtk_widget_hide (widget);
+			}
+			g_object_set_data_full (G_OBJECT (combo), "gui", gui,
 						(GDestroyNotify) g_object_unref);
 		}
 	}
@@ -536,6 +590,10 @@ gog_series_populate_editor (GogObject *gobj,
 			gog_series_populate_fill_type_combo (GOG_SERIES (series), GTK_COMBO_BOX (widget));
 			g_signal_connect (G_OBJECT (widget), "changed",
 					  G_CALLBACK (cb_fill_type_changed), series);
+			if (combo)
+				g_object_set_data (G_OBJECT (combo), "fill-type", widget);
+			if (series->interpolation == GO_LINE_INTERPOLATION_CLOSED_SPLINE)
+				gtk_widget_set_sensitive (widget, FALSE);
 			widget = glade_xml_get_widget (gui, "fill_type_prefs");
 			gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
 			g_object_set_data_full (G_OBJECT (widget), "gui", gui,
@@ -617,19 +675,25 @@ gog_series_class_init (GogSeriesClass *klass)
 			_("Has-legend"),
 			_("Should the series show up in legends"),
 			TRUE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
         g_object_class_install_property (gobject_klass, SERIES_PROP_INTERPOLATION,
 		 g_param_spec_string ("interpolation",
 			_("Interpolation"),
 			_("Type of line interpolation"),
 			"linear",
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, SERIES_PROP_INTERPOLATION_SKIP_INVALID,
+		g_param_spec_boolean ("interpolation-skip-invalid", 
+			_("Interpolation skip invalid"),
+			_("Should the series interpolation ignore the invalid data"),
+			FALSE,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_FILL_TYPE,
 		g_param_spec_string ("fill-type", 
 			_("Fill type"),
 			_("How to fill the area"),
 			"invalid",
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	klass->valid_fill_type_list = NULL;
 
@@ -1119,3 +1183,15 @@ gog_series_get_fill_type_from_combo (GogSeries const *series, GtkComboBox *combo
 	return series_klass->valid_fill_type_list[gtk_combo_box_get_active (combo)];
 }
 #endif
+
+GogDataset *
+gog_series_get_interpolation_params (GogSeries const *series)
+{
+	GogSeriesClass *series_klass;
+
+	g_return_val_if_fail (IS_GOG_SERIES (series), NULL);
+	series_klass = GOG_SERIES_GET_CLASS (series);
+	return (series_klass->get_interpolation_params)?
+			series_klass->get_interpolation_params (series):
+			NULL;
+}
