@@ -21,6 +21,7 @@
  */
 
 #include <goffice/goffice-config.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-styled-object.h>
 #include <goffice/math/go-math.h>
@@ -30,6 +31,7 @@
 #include <goffice/utils/go-image.h>
 #include <goffice/utils/go-line.h>
 #include <goffice/utils/go-marker.h>
+#include <goffice/utils/go-persist.h>
 
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
@@ -42,18 +44,9 @@
 #include <goffice/gtk/go-line-selector.h>
 #include <goffice/gtk/go-marker-selector.h>
 #include <goffice/gtk/go-pattern-selector.h>
+#include <goffice/gtk/go-image-sel.h>
 
 #include <glade/glade-xml.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkrange.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtksizegroup.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 #endif
 
@@ -115,6 +108,7 @@ typedef struct {
 	struct {
 		GtkWidget *selector;
 	} marker;
+	GODoc *doc;
 } StylePrefState;
 
 static void
@@ -462,7 +456,7 @@ static void
 cb_fill_background_color (GOSelector *selector, StylePrefState *state)
 {
 	GogStyle *style = state->style;
-	
+
 	style->fill.pattern.back = go_color_selector_get_color (selector, 
 								&style->fill.auto_back);
 	set_style (state);
@@ -514,35 +508,17 @@ fill_color_init (StylePrefState *state)
 /************************************************************************/
 
 static void
-cb_image_file_select (GtkWidget *cc, StylePrefState *state)
+cb_image_select (GtkWidget *cc, StylePrefState *state)
 {
 	GogStyle *style = state->style;
-	char *filename, *uri, *old_uri;
 	GtkWidget *w;
 
 	g_return_if_fail (style != NULL);
 	g_return_if_fail (GOG_FILL_STYLE_IMAGE == style->fill.type);
 
-	filename = style->fill.image.filename;
-	old_uri = filename ? go_filename_to_uri (filename) : NULL;
-	uri = go_gtk_select_image (GTK_WINDOW (gtk_widget_get_toplevel (cc)),
-				   old_uri);
-	g_free (old_uri);
-	if (uri == NULL)
-		return;
-	filename = go_filename_from_uri (uri);
-	g_free (uri);
-	if (filename == NULL) {
-		g_warning ("Sorry -- cannot handle URIs here right now.");
-		return;
-	}
-/* FIXME FIXME FIXME Handle URIs here. */
-
-	gog_style_set_fill_image_filename (style, filename);
-
-	w = glade_xml_get_widget (state->gui, "fill_image_sample");
-	g_object_set_data (G_OBJECT (w), "filename",
-			   style->fill.image.filename);
+	w = go_image_sel_new (state->doc, NULL, &style->fill.image.image);
+	gtk_window_set_transient_for (GTK_WINDOW (w), GTK_WINDOW (gtk_widget_get_toplevel (cc)));
+	gtk_dialog_run (GTK_DIALOG (w));
 
 	gog_style_set_image_preview (style->fill.image.image, state);
 	set_style (state);
@@ -567,7 +543,7 @@ fill_image_init (StylePrefState *state)
 	w = glade_xml_get_widget (state->gui, "fill_image_select_picture");
 	g_signal_connect (G_OBJECT (w),
 		"clicked",
-		G_CALLBACK (cb_image_file_select), state);
+		G_CALLBACK (cb_image_select), state);
 
 	sample = glade_xml_get_widget (state->gui, "fill_image_sample");
 	gtk_widget_set_size_request (sample, HSCALE + 10, VSCALE + 10);
@@ -582,8 +558,6 @@ fill_image_init (StylePrefState *state)
 		state->fill.image.image = style->fill.image.image;
 		if (state->fill.image.image)
 			g_object_ref (state->fill.image.image);
-		g_object_set_data (G_OBJECT (sample), "filename",
-				   style->fill.image.filename);
 	} else
 		gtk_combo_box_set_active (GTK_COMBO_BOX (type), 0);
 	g_signal_connect (G_OBJECT (type),
@@ -670,7 +644,8 @@ fill_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 	fill_color_init (state);
 	fill_pattern_init (state);
 	fill_gradient_init (state);
-	fill_image_init (state);
+	if (state->doc != NULL)
+		fill_image_init (state);
 	fill_update_selectors (state);
 
 	state->fill.notebook = glade_xml_get_widget (state->gui, "fill_notebook");
@@ -686,8 +661,11 @@ fill_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 				type = FILL_TYPE_GRADIENT_BICOLOR;
 			break;
 		case GOG_FILL_STYLE_IMAGE:
-			type = FILL_TYPE_IMAGE;
-			break;
+			if (state->doc != NULL) {
+				type = FILL_TYPE_IMAGE;
+				break;
+			} else
+				state->style->fill.type = GOG_FILL_STYLE_NONE;
 		case GOG_FILL_STYLE_NONE:
 		default:
 			type = FILL_TYPE_NONE;
@@ -696,6 +674,8 @@ fill_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 	fill_update_visibilies (type, state);
 
 	w = glade_xml_get_widget (state->gui, "fill_type_menu");
+	if (state->doc == NULL)
+		gtk_combo_box_remove_text (GTK_COMBO_BOX (w), FILL_TYPE_IMAGE);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (w), type);
 	g_signal_connect (G_OBJECT (w),
 		"changed",
@@ -763,6 +743,8 @@ cb_marker_size_changed (GtkAdjustment *adj, StylePrefState *state)
 	set_style (state);
 }
 
+static void gog_style_pref_state_free (StylePrefState *state);
+
 static void
 marker_init (StylePrefState *state, gboolean enable, GogEditor *editor, GOCmdContext *cc)
 {
@@ -840,6 +822,10 @@ marker_init (StylePrefState *state, gboolean enable, GogEditor *editor, GOCmdCon
 	w = glade_xml_get_widget (gui, "gog_style_marker_prefs");
 
 	gog_editor_add_page (editor, w, _("Markers"));
+	g_object_unref (gui);
+	if (state->gui == NULL)
+		g_object_set_data_full (G_OBJECT (w),
+			"state", state, (GDestroyNotify) gog_style_pref_state_free);
 }
 
 /************************************************************************/
@@ -933,7 +919,7 @@ text_layout_init (StylePrefState *state, guint32 enable, GogEditor *editor, GOCm
 	g_signal_connect (G_OBJECT (w), "rotation-changed",
 		G_CALLBACK (cb_angle_changed), state);
 	box = gtk_vbox_new (FALSE, 6);
-	gtk_box_pack_start_defaults (GTK_BOX (box), w);
+	gtk_box_pack_start (GTK_BOX (box), w, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (box), 12);
 	gog_editor_add_page (editor, box, _("Text"));
 }
@@ -958,7 +944,10 @@ gog_style_pref_state_free (StylePrefState *state)
 	}
 	g_object_unref (state->style);
 	g_object_unref (state->default_style);
-	g_object_unref (state->gui);
+	if (state->gui)
+		g_object_unref (state->gui);
+	if (state->doc)
+		g_object_unref (state->doc);
 	if (state->font_gui != NULL)
 		g_object_unref (state->font_gui);
 	if (state->fill.image.image != NULL)
@@ -978,35 +967,43 @@ gog_style_populate_editor (GogStyle *style,
 	GtkWidget *w;
 	GladeXML *gui;
 	StylePrefState *state;
+	GogGraph *graph;
 
 	g_return_if_fail (style != NULL);
 	g_return_if_fail (default_style != NULL);
 
 	enable = style->interesting_fields;
 
-	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_prefs", GETTEXT_PACKAGE, cc);
-	if (gui == NULL)
-		return;
-
 	g_object_ref (style);
 	g_object_ref (default_style);
 
 	state = g_new0 (StylePrefState, 1);
-	state->gui = gui;
+	state->gui = NULL;
 	state->font_gui = NULL;
 	state->style = style;
 	state->default_style = default_style;
 	state->object_with_style = object_with_style;
 	state->enable_edit = FALSE;
 
-	w = glade_xml_get_widget (gui, "gog_style_prefs");
-	g_object_set_data_full (G_OBJECT (w),
-		"state", state, (GDestroyNotify) gog_style_pref_state_free);
-	gog_editor_add_page (editor, w, _("Style"));
+	graph = gog_object_get_graph (GOG_OBJECT (object_with_style));
+	g_object_get (graph, "document", &state->doc, NULL);
 
-	outline_init 	 (state, enable & GOG_STYLE_OUTLINE, editor);
-	line_init    	 (state, enable & GOG_STYLE_LINE, editor);
-	fill_init    	 (state, enable & GOG_STYLE_FILL, editor);
+	if ((enable & (GOG_STYLE_OUTLINE | GOG_STYLE_LINE | GOG_STYLE_FILL)) != 0) {
+		gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_prefs", GETTEXT_PACKAGE, cc);
+		if (gui == NULL) {
+			g_free (state);
+			return;
+		}
+		state->gui = gui;
+		w = glade_xml_get_widget (gui, "gog_style_prefs");
+		g_object_set_data_full (G_OBJECT (w),
+			"state", state, (GDestroyNotify) gog_style_pref_state_free);
+		gog_editor_add_page (editor, w, _("Style"));
+
+		outline_init 	 (state, enable & GOG_STYLE_OUTLINE, editor);
+		line_init    	 (state, enable & GOG_STYLE_LINE, editor);
+		fill_init    	 (state, enable & GOG_STYLE_FILL, editor);
+	}
 	marker_init  	 (state, enable & GOG_STYLE_MARKER, editor, cc);
 	font_init    	 (state, enable & GOG_STYLE_FONT, editor, cc);
 	text_layout_init (state, enable & GOG_STYLE_TEXT_LAYOUT, editor, cc);
@@ -1084,7 +1081,6 @@ gog_style_assign (GogStyle *dst, GogStyle const *src)
 	if (GOG_FILL_STYLE_IMAGE == dst->fill.type) {
 		if (dst->fill.image.image != NULL)
 			g_object_unref (dst->fill.image.image);
-		g_free (dst->fill.image.filename);
 	}
 
 	if (src->font.font != NULL)
@@ -1101,8 +1097,8 @@ gog_style_assign (GogStyle *dst, GogStyle const *src)
 	dst->marker.mark = go_marker_dup (src->marker.mark);
 	dst->font    = src->font;
 
-	if (GOG_FILL_STYLE_IMAGE == dst->fill.type)
-		dst->fill.image.filename = g_strdup (dst->fill.image.filename);
+	if (GOG_FILL_STYLE_IMAGE == dst->fill.type && src->fill.image.image)
+		dst->fill.image.image = g_object_ref (src->fill.image.image);
 
 	dst->text_layout = src->text_layout;
 
@@ -1391,11 +1387,17 @@ gog_style_fill_sax_save (GsfXMLOut *output, GogStyle const *style)
 		gog_style_gradient_sax_save (output, style);
 		break;
 	case GOG_FILL_STYLE_IMAGE:
+		if (NULL == style->fill.image.image) {
+			g_warning ("droping fill with missing image");
+			break;
+		}
+
 		gsf_xml_out_start_element (output, "image");
 		/* FIXME : type is not a good characterization */
 		gsf_xml_out_add_cstr_unchecked (output, "type",
 			image_tiling_as_str (style->fill.image.type));
-/* TODO save the pixels */
+		gsf_xml_out_add_cstr (output, "name", go_image_get_name (style->fill.image.image));
+		go_doc_save_image ((GODoc *) g_object_get_data (G_OBJECT (gsf_xml_out_get_output (output)), "document"), go_image_get_name (style->fill.image.image));
 		gsf_xml_out_end_element (output);
 		break;
 	default:
@@ -1623,12 +1625,13 @@ static void
 gog_style_text_layout_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
 	gsf_xml_out_start_element (output, "text_layout");
-	gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 6);
+	if (!style->text_layout.auto_angle)
+		gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 6);
 	gsf_xml_out_end_element (output);
 }
 
 static gboolean
-gog_style_persist_dom_load (GogPersist *gp, xmlNode *node)
+gog_style_persist_dom_load (GOPersist *gp, xmlNode *node)
 {
 	GogStyle *style = GOG_STYLE (gp);
 	xmlNode *ptr;
@@ -1712,13 +1715,15 @@ static void
 gog_style_sax_load_fill_image (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GogStyle *style = GOG_STYLE (xin->user_state);
+	GODoc *doc = (GODoc *) g_object_get_data (G_OBJECT (gsf_xml_in_get_input (xin)), "document");
 	g_return_if_fail (style->fill.type == GOG_FILL_STYLE_IMAGE);
+	g_return_if_fail (IS_GO_DOC (doc));
 	/* TODO: load the pixels */
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (0 == strcmp (attrs[0], "type")) {
 			style->fill.image.type = str_as_image_tiling (attrs[1]);
-			break;
-		}
+		} else if (0 == strcmp (attrs[0], "name"))
+			style->fill.image.image = g_object_ref (go_doc_image_fetch (doc, attrs[1]));
 }
 
 static void
@@ -1788,7 +1793,7 @@ gog_style_sax_load_text_layout (GsfXMLIn *xin, xmlChar const **attrs)
 }
 
 static void
-gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
+gog_style_persist_sax_save (GOPersist const *gp, GsfXMLOut *output)
 {
 	GogStyle const *style = GOG_STYLE (gp);
 
@@ -1810,7 +1815,7 @@ gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
 }
 
 static void
-gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
+gog_style_persist_prep_sax (GOPersist *gp, GsfXMLIn *xin, xmlChar const **attrs)
 {
 	static GsfXMLInNode const dtd[] = {
 		GSF_XML_IN_NODE 	(STYLE, STYLE, 
@@ -1838,7 +1843,7 @@ gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs
 					 &gog_style_sax_load_fill_gradient, NULL),
 		GSF_XML_IN_NODE 	(STYLE_FILL, FILL_IMAGE, 
 					 -1, "image", 
-					 GSF_XML_CONTENT, 
+					 GSF_XML_NO_CONTENT, 
 					 &gog_style_sax_load_fill_image, NULL),
 		GSF_XML_IN_NODE 	(STYLE, STYLE_MARKER,	
 					 -1, "marker", 
@@ -1862,7 +1867,7 @@ gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs
 }
 
 static void
-gog_style_persist_init (GogPersistClass *iface)
+gog_style_persist_init (GOPersistClass *iface)
 {
 	iface->dom_load = gog_style_persist_dom_load;
 	iface->prep_sax = gog_style_persist_prep_sax;
@@ -1872,7 +1877,7 @@ gog_style_persist_init (GogPersistClass *iface)
 GSF_CLASS_FULL (GogStyle, gog_style,
 		NULL, NULL, gog_style_class_init, NULL,
 		gog_style_init, G_TYPE_OBJECT, 0,
-		GSF_INTERFACE (gog_style_persist_init, GOG_PERSIST_TYPE))
+		GSF_INTERFACE (gog_style_persist_init, GO_PERSIST_TYPE))
 
 gboolean
 gog_style_is_different_size (GogStyle const *a, GogStyle const *b)
@@ -2022,31 +2027,6 @@ gog_style_set_fill_brightness (GogStyle *style, float brightness)
 		: UINT_INTERPOLATE(style->fill.pattern.back, RGBA_BLACK, brightness / 50. - 1.);
 }
 
-/**
- * gog_style_set_fill_image_filename :
- * @style : #GogStyle
- * @filename :
- *
- * absorb the string and eventually free it.
- **/
-void
-gog_style_set_fill_image_filename (GogStyle *style, char *filename)
-{
-	g_return_if_fail (IS_GOG_STYLE (style));
-
-	if (style->fill.type == GOG_FILL_STYLE_IMAGE) {
-		if (style->fill.image.image != NULL)
-			g_object_unref (style->fill.image.image);
-		g_free (style->fill.image.filename);
-	} else {
-		style->fill.type = GOG_FILL_STYLE_IMAGE;
-		style->fill.image.type = GOG_IMAGE_CENTERED;
-	}
-
-	style->fill.image.filename = filename;
-	style->fill.image.image = go_image_new_from_file (filename, NULL);
-}
-
 /**
  * gog_style_set_text_angle:
  * @style : #GogStyle
@@ -2136,6 +2116,11 @@ gog_style_create_cairo_pattern (GogStyle const *style, cairo_t *cr)
 				return cairo_pattern_create_rgba (1, 1, 1, 1);
 
 			cr_pattern = go_image_create_cairo_pattern (style->fill.image.image);
+			if (cr_pattern == NULL) {
+				/* don't reference anymore an invalid image */
+				((GogStyle *) style)->fill.image.image = NULL;
+				return cairo_pattern_create_rgba (1, 1, 1, 1);
+			}
 			g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
 			switch (style->fill.image.type) {
