@@ -22,11 +22,16 @@
 
 
 /**
+ * go_xml_parse_file :
+ * @filename: the locale path to a file to parse.
+ *
  * Like xmlParseFile, but faster.  Does not accept compressed files.
  * See http://bugzilla.gnome.org/show_bug.cgi?id=168414
  *
  * Note: this reads the entire file into memory and should therefore
  * not be used for user-supplied files.
+ *
+ * Returns: A libxml2 xmlDocPtr or %NULL.
  **/
 xmlDocPtr
 go_xml_parse_file (char const *filename)
@@ -279,7 +284,7 @@ e_xml_get_child_by_name_by_lang (xmlNode const *parent, gchar const *name)
 		if (node->name == NULL || strcmp (CXML2C (node->name), name) != 0)
 			continue;
 
-		lang = xmlGetProp (node, CC2XML ("xml:lang"));
+		lang = xmlGetProp (node, CC2XML ("lang"));
 		if (lang != NULL) {
 			gint i;
 
@@ -289,10 +294,10 @@ e_xml_get_child_by_name_by_lang (xmlNode const *parent, gchar const *name)
 					best_lang_score = i;
 				}
 			}
+			xmlFree (lang);
 		} else if (best_node == NULL)
 			best_node = node;
 
-		xmlFree (lang);
 		if (best_lang_score == 0) 
 			return best_node;
 	}
