@@ -236,7 +236,6 @@ gog_plot1_5d_update (GogObject *obj)
 			series = ptr->data;
 			/* we are guaranteed that at least 1 series is valid above */
 			if (!gog_series_is_valid (GOG_SERIES (series))) {
-				lengths[i] = 0;
 				continue;
 			}
 			vals[i] = go_data_vector_get_values (
@@ -359,7 +358,6 @@ gog_plot1_5d_class_init (GogPlotClass *plot_klass)
 		plot_klass->desc.series.dim = dimensions;
 		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
 	}
-	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_series1_5d_get_type ();
 	plot_klass->axis_get_bounds   	= gog_plot1_5d_axis_get_bounds;
@@ -558,7 +556,7 @@ gog_series1_5d_get_property (GObject *obj, guint param_id,
 }
 
 #ifdef GOFFICE_WITH_GTK
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 static void 
 gog_series1_5d_populate_editor (GogObject *obj,
 				GogEditor *editor,
