@@ -31,10 +31,47 @@
 #include <goffice/data/go-data.h>
 #include <goffice/math/go-math.h>
 #include <goffice/utils/go-color.h>
+#include <goffice/utils/go-persist.h>
 
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 
+/******************************************************************************/
+
+typedef GogSeriesElement GogBarColSeriesElement;
+typedef GogSeriesElementClass GogBarColSeriesElementClass;
+
+#define GOG_BARCOL_SERIES_ELEMENT_TYPE	(gog_barcol_series_element_get_type ())
+#define GOG_BARCOL_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_BARCOL_SERIES_ELEMENT_TYPE, GogBarColSeriesElement))
+#define GOG_IS_BARCOL_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_BARCOL_SERIES_ELEMENT_TYPE))
+GType gog_barcol_series_element_get_type (void);
+
+GSF_DYNAMIC_CLASS (GogBarColSeriesElement, gog_barcol_series_element,
+	NULL, NULL,
+	GOG_SERIES_ELEMENT_TYPE)
+
+/******************************************************************************/
+
+typedef GogSeries1_5d GogBarColSeries;
+typedef GogSeries1_5dClass GogBarColSeriesClass;
+
+#define GOG_BARCOL_SERIES_TYPE	(gog_barcol_series_eget_type ())
+#define GOG_BARCOL_SERIES(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_BARCOL_SERIES_TYPE, GogBarColSeries))
+#define GOG_IS_BARCOL_SERIEST(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_BARCOL_SERIES_TYPE))
+GType gog_barcol_series_get_type (void);
+
+static void
+gog_barcol_series_class_init (GogSeriesClass *series_klass)
+{
+	series_klass->series_element_type = GOG_BARCOL_SERIES_ELEMENT_TYPE;
+}
+
+GSF_DYNAMIC_CLASS (GogBarColSeries, gog_barcol_series,
+	gog_barcol_series_class_init, NULL,
+	GOG_SERIES1_5D_TYPE)
+
+/******************************************************************************/
+
 enum {
 	BARCOL_PROP_0,
 	BARCOL_PROP_GAP_PERCENTAGE,
@@ -204,25 +241,26 @@ gog_barcol_plot_class_init (GogPlot1_5dClass *gog_plot_1_5d_klass)
 			_("Gap percentage"),
 			_("The padding around each group as a percentage of their width"),
 			0, 500, 150, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, BARCOL_PROP_OVERLAP_PERCENTAGE,
 		g_param_spec_int ("overlap-percentage", 
 			_("Overlap percentage"),
 			_("The distance between series as a percentage of their width"),
 			-100, 100, 0, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, BARCOL_PROP_HORIZONTAL,
 		g_param_spec_boolean ("horizontal", 
 			_("horizontal"),
 			_("horizontal bars or vertical columns"),
 			FALSE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	gog_object_klass->type_name	= gog_barcol_plot_type_name;
 	gog_object_klass->populate_editor	= gog_barcol_plot_populate_editor;
 	gog_object_klass->view_type	= gog_barcol_view_get_type ();
 
 	plot_klass->desc.series.style_fields	= GOG_STYLE_OUTLINE | GOG_STYLE_FILL;
+	plot_klass->series_type = gog_barcol_series_get_type ();
 	plot_klass->axis_get_bounds   		= gog_barcol_axis_get_bounds;
 
 	gog_plot_1_5d_klass->swap_x_and_y = gog_barcol_swap_x_and_y;
@@ -265,7 +303,7 @@ barcol_draw_rect (GogRenderer *rend, gboolean flip,
 		  GogAxisMap *y_map,
 		  GogViewAllocation const *rect)
 {
-	ArtVpath path[6];
+	GOPath *path = go_path_new ();
 	double x0, x1, y0, y1;
 
 	if (flip) {
@@ -292,19 +330,15 @@ barcol_draw_rect (GogRenderer *rend, gboolean flip,
 		y1 = gog_axis_map_to_view (y_map, rect->y + rect->h);
 	}
 
-	path[0].x = path[3].x = path[4].x = x0;
-	path[1].x = path[2].x = x1;
-	path[0].y = path[1].y = path[4].y = y0;
-	path[2].y = path[3].y = y1;
-	path[0].code = ART_MOVETO;
-	path[1].code = ART_LINETO;
-	path[2].code = ART_LINETO;
-	path[3].code = ART_LINETO;
-	path[4].code = ART_LINETO;
-	path[5].code = ART_END;
-	
-	gog_renderer_draw_sharp_polygon (rend, path, 
-					 fabs (x1 - x0) < 3. || fabs (y1 - y0) < 3.);
+	go_path_move_to (path, x0, y0);
+	go_path_line_to (path, x1, y0);
+	go_path_line_to (path, x1, y1);
+	go_path_line_to (path, x0, y1);
+	go_path_close (path);
+	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
+
+	gog_renderer_draw_shape (rend, path);
+	go_path_free (path);
 }
 
 typedef struct {
@@ -326,7 +360,7 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogViewAllocation const *area;
 	GogRenderer *rend = view->renderer;
 	GogAxisMap *x_map, *y_map, *map;
-	gboolean is_vertical = ! (model->horizontal);
+	gboolean is_vertical = ! (model->horizontal), valid;
 	double **vals, sum, neg_base, pos_base, tmp;
 	double x;
 	double col_step, group_step, offset, data_scale;
@@ -338,12 +372,13 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	ErrorBarData **error_data;
 	GogErrorBar **errors;
 	GogSeriesLines **lines;
-	ArtVpath **paths;
+	GOPath **paths;
 	GSList *ptr;
 	unsigned *lengths;
 	double plus, minus;
 	GogObjectRole const *role = NULL;
-
+	GogSeriesElement *gse;
+	GList const *overrides;
 
 	if (num_elements <= 0 || num_series <= 0)
 		return;
@@ -369,7 +404,7 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	errors = g_alloca (num_series * sizeof (GogErrorBar *));
 	error_data = g_alloca (num_series * sizeof (ErrorBarData *));
 	lines = g_alloca (num_series * sizeof (GogSeriesLines *));
-	paths = g_alloca (num_series * sizeof (ArtVpath *));
+	paths = g_alloca (num_series * sizeof (GOPath *));
 	
 	i = 0;
 	for (ptr = gog_1_5d_model->base.series ; ptr != NULL ; ptr = ptr->next) {
@@ -386,18 +421,13 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 			error_data[i] = g_malloc (sizeof (ErrorBarData) * lengths[i]);
 		else 
 			error_data[i] = NULL;
-		if (series->has_series_lines) {
+		if (series->has_series_lines && lengths[i] > 0) {
 			if (!role)
 				role = gog_object_find_role_by_name (
 							GOG_OBJECT (series), "Series lines");
 			lines[i] = GOG_SERIES_LINES (
 					gog_object_get_child_by_role (GOG_OBJECT (series), role));
-			paths[i] = g_new (ArtVpath, lengths[i] * 2 - 1);
-			for (j = 0; j < lengths[i] - 1; j++) {
-				paths[i][j * 2].code = ART_MOVETO;
-				paths[i][j * 2 + 1].code = ART_LINETO;
-			}
-			paths[i][j * 2].code = ART_END;
+			paths[i] = go_path_new ();
 		} else
 			lines[i] = NULL;
 		i++;
@@ -430,16 +460,19 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 		}
 
 		pos_base = neg_base = 0.0;
+		ptr = gog_1_5d_model->base.series;
 		for (j = 0 ; j < num_series ; j++) {
-			
+			overrides = gog_series_get_overrides (GOG_SERIES (ptr->data));
 			work.y = (double) j * col_step + (double) i - offset + 1.0;
 			
 			if (i >= lengths[j])
 				continue;
 			tmp = vals[j][i];
-			if (!gog_axis_map_finite (map, tmp))
-				continue;
-			if (gog_error_bar_is_visible (errors[j])) {
+			valid = TRUE;
+			if (!gog_axis_map_finite (map, tmp)) {
+				tmp = 0;
+				valid = FALSE;
+			} else if (gog_error_bar_is_visible (errors[j])) {
 				gog_error_bar_get_bounds (errors[j], i, &minus, &plus);
 			}
 			tmp *= data_scale;
@@ -455,11 +488,20 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 					neg_base += tmp;
 			}
 
-			gog_renderer_push_style (view->renderer, styles[j]);
+			gse = NULL;
+			if ((overrides != NULL) &&
+				(GOG_SERIES_ELEMENT (overrides->data)->index == i)) {
+					gse = GOG_SERIES_ELEMENT (overrides->data);
+					overrides = overrides->next;
+					gog_renderer_push_style (view->renderer,
+						gog_styled_object_get_style (
+							GOG_STYLED_OBJECT (gse)));
+			} else
+				gog_renderer_push_style (view->renderer, styles[j]);
 			barcol_draw_rect (rend, is_vertical, x_map, y_map, &work);
 			gog_renderer_pop_style (view->renderer);
 			
-			if (gog_error_bar_is_visible (errors[j])) {
+			if (valid && gog_error_bar_is_visible (errors[j])) {
 				x = tmp > 0 ? work.x + work.w: work.x;
 				error_data[j][i].plus = plus * data_scale;
 				error_data[j][i].minus = minus * data_scale;
@@ -475,20 +517,25 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 				x = tmp > 0 ? work.x + work.w: work.x;
 				if (is_vertical) {
 					if (i > 0) {
-						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, work.y);
-						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, x);
+						go_path_line_to (paths[j],
+								 gog_axis_map_to_view (x_map, work.y),
+								 gog_axis_map_to_view (y_map, x));
 					}
-					paths[j][i * 2].x = gog_axis_map_to_view (x_map, work.y + work.h);
-					paths[j][i * 2].y = gog_axis_map_to_view (y_map, x);
+					go_path_move_to (paths[j],
+							 gog_axis_map_to_view (x_map, work.y + work.h),
+							 gog_axis_map_to_view (y_map, x));
 				} else {
 					if (i > 0) {
-						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, x);
-						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, work.y);
+						go_path_line_to (paths[j],
+								 gog_axis_map_to_view (x_map, x),
+								 gog_axis_map_to_view (y_map, work.y));
 					}
-					paths[j][i * 2].x = gog_axis_map_to_view (x_map, x);
-					paths[j][i * 2].y = gog_axis_map_to_view (y_map, work.y + work.h);
+					go_path_move_to (paths[j],
+							 gog_axis_map_to_view (x_map, x),
+							 gog_axis_map_to_view (y_map, work.y + work.h));
 				}
 			}
+			ptr = ptr->next;
 		}
 	}
 	/*Now draw error bars and clean*/
@@ -506,8 +553,8 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	/* Draw series lines if any */
 	for (i = 0; i < num_series; i++)
 		if (lines[i] != NULL) {
-			gog_series_lines_render (lines[i], view->renderer, bbox, paths[i], FALSE);
-			g_free (paths[i]);
+			gog_series_lines_stroke (lines[i], view->renderer, bbox, paths[i], FALSE);
+			go_path_free (paths[i]);
 		}
 
 	gog_chart_map_free (chart_map);
