@@ -58,10 +58,6 @@ static GType gog_box_plot_view_get_type (void);
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
 #include <glade/glade-xml.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkenums.h>
 
 static void
 cb_gap_changed (GtkAdjustment *adj, GObject *boxplot)
@@ -398,7 +394,6 @@ gog_box_plot_class_init (GogPlotClass *gog_plot_klass)
 		plot_klass->desc.series.dim = dimensions;
 		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
 	}
-	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_box_plot_series_get_type ();
 	plot_klass->axis_set = GOG_AXIS_SET_XY;
