@@ -31,11 +31,13 @@
 #include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/data/go-data.h>
+#include <goffice/data/go-data-simple.h>
 #include <goffice/utils/go-color.h>
 #include <goffice/utils/go-marker.h>
 #include <goffice/utils/go-format.h>
 #include <goffice/math/go-math.h>
 #include <goffice/utils/go-line.h>
+#include <goffice/utils/go-persist.h>
 #include <goffice/app/module-plugin-defs.h>
 
 #ifdef GOFFICE_WITH_GTK
@@ -231,7 +233,6 @@ gog_2d_plot_class_init (GogPlotClass *plot_klass)
 	gog_klass->update	= gog_2d_plot_update;
 	gog_klass->view_type	= gog_xy_view_get_type ();
 
-	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type  	= gog_xy_series_get_type ();
 	plot_klass->axis_set	      	= GOG_AXIS_SET_XY;
@@ -342,25 +343,25 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 			_("Has markers by default"),
 			_("Should the default style of a series include markers"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES,
 		g_param_spec_boolean ("default-style-has-lines", 
 			_("Has lines by default"),
 			_("Should the default style of a series include lines"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_DEFAULT_STYLE_HAS_FILL,
 		g_param_spec_boolean ("default-style-has-fill", 
 			_("Has fill by default"),
 			_("Should the default style of a series include fill"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_USE_SPLINES,
 		g_param_spec_boolean ("use-splines",
 			_("Use splines"),
 			_("Should the plot use splines instead of linear interpolation"),
 			FALSE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	gog_klass->type_name	= gog_xy_plot_type_name;
 
 	{
@@ -511,25 +512,25 @@ gog_bubble_plot_class_init (GogPlotClass *plot_klass)
 			_("Size as area"),
 			_("Display size as area instead of diameter"),
 			TRUE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_SHOW_NEGATIVES,
 		g_param_spec_boolean ("show-negatives", 
 			_("Show negatives"),
 			_("Draw bubbles for negative values"),
 			FALSE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_IN_3D,
 		g_param_spec_boolean ("in-3d", 
 			_("In 3d"),
 			_("Draw 3d bubbles"),
 			FALSE,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_SCALE,
 		g_param_spec_float ("bubble-scale", 
 			_("Bubble scale"),
 			_("Fraction of default radius used for display"),
-			0., 2., 1.,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			0., 3., 1.,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	{
 		static GogSeriesDimDesc dimensions[] = {
 			{ N_("X"), GOG_SERIES_SUGGESTED, FALSE,
@@ -554,7 +555,7 @@ gog_bubble_plot_class_init (GogPlotClass *plot_klass)
 	}
 }
 
-#define BUBBLE_MAX_RADIUS_RATIO 8.
+#define BUBBLE_MAX_RADIUS_RATIO 12.	/* 4 * MAX(bubble-scale) */
 static void
 gog_bubble_plot_adjust_bounds (Gog2DPlot *model, double *x_min, double *x_max, double *y_min, double *y_max)
 {
@@ -771,19 +772,19 @@ gog_xy_color_plot_class_init (GogPlotClass *plot_klass)
 			_("Has lines by default"),
 			_("Should the default style of a series include lines"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_FILL,
 		g_param_spec_boolean ("default-style-has-fill", 
 			_("Has fill by default"),
 			_("Should the default style of a series include fill"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_HIDE_OUTLIERS,
 		g_param_spec_boolean  ("hide-outliers", 
 			_("hide-outliers"),
 			_("Hide data outside of the color axis bounds"),
 			TRUE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	gog_klass->type_name		= gog_xy_color_plot_type_name;
 	gog_klass->update		= gog_xy_color_plot_update;
 	gog_klass->populate_editor	= gog_xy_color_plot_populate_editor;
@@ -882,6 +883,9 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 	MarkerData **markers;
 	unsigned *num_markers;
 
+	GogSeriesElement *gse;
+	GList const *overrides;
+
 	for (num_series = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, num_series++);
 	if (num_series < 1)
 		return;
@@ -904,12 +908,10 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		hide_outliers = GOG_XY_COLOR_PLOT (model)->hide_outliers;
 	}
 
-	/* Draw drop lines from point to axis start. To change this behaviour
-	 * and draw drop lines from point to zero, we can use gog_axis_map_get_baseline:
-	 * x_bottom = gog_axis_map_get_baseline (x_map); 
-	 * What we really want is to draw drop lines from point to
+	/* What we really want is to draw drop lines from point to
 	 * a selected axis. But for this purpose, we need a GogAxisBase selector in
-	 * GogSeriesLine, which doesn't really know what it's supposed to do with it. */
+	 * GogSeriesLines (we might actually need GogDropLines. For now just let
+	 * the drop lines go the the axis. */
 
 	gog_axis_map_get_extents (x_map, &x_left, &x_right);
 	x_left = gog_axis_map_to_view (x_map, x_left);
@@ -956,11 +958,15 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 				path = next_path;
 			else
 				path = gog_chart_map_make_path (chart_map, x_vals, y_vals,
-								n, series->base.interpolation);
+								n, series->base.interpolation,
+								series->base.interpolation_skip_invalid,
+								&series->clamped_derivs);
 
 			next_path = NULL;
 
-			if (series->base.fill_type != GOG_SERIES_FILL_TYPE_NEXT) {
+			if (series->base.interpolation == GO_LINE_INTERPOLATION_CLOSED_SPLINE)
+				gog_renderer_fill_serie	(view->renderer, path, NULL);
+			else if (series->base.fill_type != GOG_SERIES_FILL_TYPE_NEXT) {
 				GOPath *close_path;
 
 				close_path = gog_chart_map_make_close_path (chart_map, x_vals, y_vals, n,
@@ -986,7 +992,9 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 
 						next_path = gog_chart_map_make_path
 							(chart_map, next_x_vals, next_y_vals,
-							 next_n_points, next_series->base.interpolation);
+							 next_n_points, next_series->base.interpolation,
+							 series->base.interpolation_skip_invalid,
+							 &series->clamped_derivs);
 
 					}
 				}
@@ -996,8 +1004,32 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 
 		if (series->hdroplines) {
 			GOPath *drop_path;
-			double y_drop;
-
+			double y_drop, x_target;
+			GogAxis *axis = GOG_PLOT (model)->axis[GOG_AXIS_Y];
+			GogAxisPosition pos = gog_axis_base_get_clamped_position (GOG_AXIS_BASE (axis));
+			switch (pos) {
+			case GOG_AXIS_AT_LOW:
+				x_target = gog_axis_map_is_inverted (x_map)? x_right: x_left;
+				break;
+			case GOG_AXIS_CROSS: {
+				GogChartMap *c_map;
+				GogAxisMap *a_map;
+				c_map = gog_chart_map_new (chart, area, axis,
+						gog_axis_base_get_crossed_axis (GOG_AXIS_BASE (axis)),
+						NULL, FALSE);
+				a_map = gog_chart_map_get_axis_map (c_map, 1);
+				x_target = gog_axis_map_to_view (a_map, gog_axis_base_get_cross_location (GOG_AXIS_BASE (axis)));
+				gog_chart_map_free (c_map);
+				break;
+			}
+			case GOG_AXIS_AT_HIGH:
+				x_target = gog_axis_map_is_inverted (x_map)? x_left: x_right;
+				break;
+			default:
+				/* this should not occur */
+				x_target = gog_axis_map_to_view (x_map, 0);
+				break;
+			}
 			gog_renderer_push_style (view->renderer,
 				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->hdroplines)));
 			drop_path = go_path_new ();
@@ -1009,7 +1041,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 				if (!gog_axis_map_finite (x_map, x))
 					continue;
 				y_drop = gog_axis_map_to_view (y_map, y_vals[i]);
-				go_path_move_to (drop_path, x_left, y_drop);
+				go_path_move_to (drop_path, x_target, y_drop);
 				go_path_line_to (drop_path, gog_axis_map_to_view (x_map, x), y_drop);
 			}
 			gog_renderer_stroke_serie (view->renderer, drop_path);
@@ -1019,8 +1051,32 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 
 		if (series->vdroplines) {
 			GOPath *drop_path;
-			double x_drop;
-
+			double x_drop, y_target;
+			GogAxis *axis = GOG_PLOT (model)->axis[GOG_AXIS_X];
+			GogAxisPosition pos = gog_axis_base_get_clamped_position (GOG_AXIS_BASE (axis));
+			switch (pos) {
+			case GOG_AXIS_AT_LOW:
+				y_target = gog_axis_map_is_inverted (y_map)? y_top: y_bottom;
+				break;
+			case GOG_AXIS_CROSS: {
+				GogChartMap *c_map;
+				GogAxisMap *a_map;
+				c_map = gog_chart_map_new (chart, area, axis,
+						gog_axis_base_get_crossed_axis (GOG_AXIS_BASE (axis)),
+						NULL, FALSE);
+				a_map = gog_chart_map_get_axis_map (c_map, 1);
+				y_target = gog_axis_map_to_view (a_map, gog_axis_base_get_cross_location (GOG_AXIS_BASE (axis)));
+				gog_chart_map_free (c_map);
+				break;
+			}
+			case GOG_AXIS_AT_HIGH:
+				y_target = gog_axis_map_is_inverted (y_map)? y_bottom: y_top;
+				break;
+			default:
+				/* this should not occur */
+				y_target = gog_axis_map_to_view (y_map, 0);
+				break;
+			}
 			gog_renderer_push_style (view->renderer,
 				gog_styled_object_get_style (GOG_STYLED_OBJECT (series->vdroplines)));
 			drop_path = go_path_new ();
@@ -1032,7 +1088,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 				if (!gog_axis_map_finite (x_map, x))
 					continue;
 				x_drop = gog_axis_map_to_view (x_map, x);
-				go_path_move_to (drop_path, x_drop, y_bottom);
+				go_path_move_to (drop_path, x_drop, y_target);
 				go_path_line_to (drop_path, x_drop, gog_axis_map_to_view (y_map, y_vals[i]));
 			}
 			gog_renderer_stroke_serie (view->renderer, drop_path);
@@ -1073,6 +1129,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 		y_margin_min = view->allocation.y - margin;
 		y_margin_max = view->allocation.y + view->allocation.h + margin;
 
+		overrides = gog_series_get_overrides (GOG_SERIES (series));
 		k = 0;
 		for (i = 1 ; i <= n ; i++) {
 			x = x_vals ? *x_vals++ : i;
@@ -1104,13 +1161,23 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 						gog_renderer_pop_style (view->renderer);
 					} else continue;
 				} else {
-					if (model->base.vary_style_by_element)
+					gse = NULL;
+					if ((overrides != NULL) &&
+					   	(GOG_SERIES_ELEMENT (overrides->data)->index == i - 1)) {
+							gse = GOG_SERIES_ELEMENT (overrides->data);
+							overrides = overrides->next;
+							gog_renderer_push_style (view->renderer,
+								gog_styled_object_get_style (
+									GOG_STYLED_OBJECT (gse)));
+					} else if (model->base.vary_style_by_element)
 						gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
 									model->base.index_num + i - 1, FALSE);
 					gog_renderer_draw_circle (view->renderer, x_canvas, y_canvas, 
 							    ((size_as_area) ?
 							    sqrt (z / zmax) :
 							    z / zmax) * rmax);
+					if (gse)
+						gog_renderer_pop_style (view->renderer);
 				}
 			}
 
@@ -1166,11 +1233,21 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	if (!GOG_IS_BUBBLE_PLOT (model))
 		for (j = 0, ptr = model->base.series ; ptr != NULL ; ptr = ptr->next, j++) {
+			overrides = gog_series_get_overrides (GOG_SERIES (series));
 				if (markers[j] != NULL) {
 					series = ptr->data;
+					overrides = gog_series_get_overrides (GOG_SERIES (series));
 					style = GOG_STYLED_OBJECT (series)->style;
 					gog_renderer_push_style (view->renderer, style);
 					for (k = 0; k < num_markers[j]; k++) {
+						gse = NULL;
+						if ((overrides != NULL) &&
+							(GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
+								gse = GOG_SERIES_ELEMENT (overrides->data);
+								overrides = overrides->next;
+								style = gog_styled_object_get_style (GOG_STYLED_OBJECT (gse));
+								gog_renderer_push_style (view->renderer, style);
+						}
 						if (is_map) {
 							go_marker_set_outline_color
 								(style->marker.mark,markers[j][k].color);
@@ -1183,6 +1260,10 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 									  markers[j][k].y);
 						if (is_map)
 							gog_renderer_pop_style (view->renderer);
+						if (gse) {
+							gog_renderer_pop_style (view->renderer);
+							style = GOG_STYLED_OBJECT (series)->style;
+						}
 					}
 					gog_renderer_pop_style (view->renderer);
 					g_free (markers[j]);
@@ -1311,17 +1392,144 @@ vert_drop_lines_pre_remove (GogObject *parent, GogObject *child)
 
 /****************************************************************************/
 
+typedef GogSeriesElement GogXYSeriesElement;
+typedef GogSeriesElementClass GogXYSeriesElementClass;
+#define GOG_XY_SERIES_ELEMENT_TYPE	(gog_xy_series_element_get_type ())
+#define GOG_XY_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_XY_SERIES_ELEMENT_TYPE, GogXYSeriesElement))
+#define IS_GOG_XY_SERIES_ELEMENT(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_XY_SERIES_ELEMENT_TYPE))
+GType gog_xy_series_element_get_type (void);
+
+static void
+gog_xy_series_element_init_style (GogStyledObject *gso, GogStyle *style)
+{
+	GogSeries const *series = GOG_SERIES (GOG_OBJECT (gso)->parent);
+	GogStyle *parent_style;
+
+	g_return_if_fail (series != NULL);
+
+	parent_style = gog_styled_object_get_style (GOG_STYLED_OBJECT (series));
+	if (parent_style->interesting_fields & GOG_STYLE_MARKER)
+		style->interesting_fields = parent_style->interesting_fields & (GOG_STYLE_MARKER | GOG_STYLE_MARKER_NO_COLOR);
+	else
+		style->interesting_fields = parent_style->interesting_fields;
+	gog_theme_fillin_style (gog_object_get_theme (GOG_OBJECT (gso)),
+		style, GOG_OBJECT (gso), GOG_SERIES_ELEMENT (gso)->index, FALSE);
+}
+
+static void
+gog_xy_series_element_class_init (GogXYSeriesElementClass *klass)
+{
+	GogStyledObjectClass *style_klass = (GogStyledObjectClass *) klass;
+	style_klass->init_style	    	= gog_xy_series_element_init_style;
+}
+
+GSF_DYNAMIC_CLASS (GogXYSeriesElement, gog_xy_series_element,
+	gog_xy_series_element_class_init, NULL,
+	GOG_SERIES_ELEMENT_TYPE)
+
+/****************************************************************************/
+
+typedef struct {
+	GogObject base;
+	GogXYSeries *series;
+	GogDatasetElement *derivs;
+} GogXYInterpolationClamps;
+
+typedef GogObjectClass GogXYInterpolationClampsClass;
+
+#define GOG_XY_INTERPOLATION_CLAMPS_TYPE	(gog_xy_interpolation_clamps_get_type ())
+#define GOG_XY_INTERPOLATION_CLAMPS(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_XY_INTERPOLATION_CLAMPS_TYPE, GogXYInterpolationClamps))
+#define GOG_IS_XY_INTERPOLATION_CLAMPS(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_XY_INTERPOLATION_CLAMPS_TYPE))
+GType gog_xy_interpolation_clamps_get_type (void);
+
+static GObjectClass *interp_parent_klass;
+
+static void
+gog_xy_interpolation_clamps_dataset_dims (GogDataset const *set, int *first, int *last)
+{
+	*first = 0;
+	*last = 1;
+}
+
+static GogDatasetElement *
+gog_xy_interpolation_clamps_dataset_get_elem (GogDataset const *set, int dim_i)
+{
+	GogXYInterpolationClamps *clamps = GOG_XY_INTERPOLATION_CLAMPS (set);
+	g_return_val_if_fail (2 > dim_i, NULL);
+	g_return_val_if_fail (dim_i >= 0, NULL);
+	return clamps->derivs + dim_i;
+}
+
+static void
+gog_xy_interpolation_clamps_dataset_dim_changed (GogDataset *set, int dim_i)
+{
+	GogXYInterpolationClamps *clamps = GOG_XY_INTERPOLATION_CLAMPS (set);
+	clamps->series->clamped_derivs[dim_i] = (IS_GO_DATA_SCALAR ((clamps->derivs + dim_i)->data))?
+		go_data_scalar_get_value (GO_DATA_SCALAR ((clamps->derivs + dim_i)->data)): 0.;
+	gog_object_request_update (GOG_OBJECT (clamps->series));
+}
+
+static void
+gog_xy_interpolation_clamps_dataset_init (GogDatasetClass *iface)
+{
+	iface->get_elem	   = gog_xy_interpolation_clamps_dataset_get_elem;
+	iface->dims	   = gog_xy_interpolation_clamps_dataset_dims;
+	iface->dim_changed = gog_xy_interpolation_clamps_dataset_dim_changed;
+}
+
+static void
+gog_xy_interpolation_clamps_finalize (GObject *obj)
+{
+	GogXYInterpolationClamps *clamps = GOG_XY_INTERPOLATION_CLAMPS (obj);
+	if (clamps->derivs != NULL) {
+		gog_dataset_finalize (GOG_DATASET (obj));
+		g_free (clamps->derivs);
+		clamps->derivs = NULL;
+	}
+	(*interp_parent_klass->finalize) (obj);
+}
+
+static void
+gog_xy_interpolation_clamps_class_init (GObjectClass *klass)
+{
+	interp_parent_klass = g_type_class_peek_parent (klass);
+	klass->finalize	    = gog_xy_interpolation_clamps_finalize;
+}
+
+static void
+gog_xy_interpolation_clamps_init (GogXYInterpolationClamps *clamps)
+{
+	clamps->derivs = g_new0 (GogDatasetElement, 2);
+}
+
+GSF_CLASS_FULL (GogXYInterpolationClamps, gog_xy_interpolation_clamps,
+		NULL, NULL, gog_xy_interpolation_clamps_class_init, NULL,
+		gog_xy_interpolation_clamps_init, GOG_OBJECT_TYPE, 0,
+		GSF_INTERFACE (gog_xy_interpolation_clamps_dataset_init, GOG_DATASET_TYPE))
+
+/****************************************************************************/
+
 typedef GogSeriesClass GogXYSeriesClass;
 
 enum {
 	SERIES_PROP_0,
 	SERIES_PROP_XERRORS,
 	SERIES_PROP_YERRORS,
-	SERIES_PROP_INVALID_AS_ZERO
+	SERIES_PROP_INVALID_AS_ZERO,
+	SERIES_PROP_CLAMP0,
+	SERIES_PROP_CLAMP1
 };
 
 static GogStyledObjectClass *series_parent_klass;
 
+static GogDataset *
+gog_xy_series_get_interpolation_params (GogSeries const *series)
+{
+	GogXYSeries *xy = GOG_XY_SERIES (series);
+	g_return_val_if_fail (xy, NULL);
+	return xy->interpolation_props;
+}
+
 static void
 gog_xy_series_update (GogObject *obj)
 {
@@ -1360,6 +1568,10 @@ gog_xy_series_init (GObject *obj)
 	(GOG_SERIES (series))->acceptable_children = GOG_SERIES_ACCEPT_TREND_LINE;
 	series->x_errors = series->y_errors = NULL;
 	series->hdroplines = series->vdroplines = NULL;
+	series->interpolation_props = g_object_new (GOG_XY_INTERPOLATION_CLAMPS_TYPE, NULL);
+	GOG_XY_INTERPOLATION_CLAMPS (series->interpolation_props)->series = series;
+	gog_dataset_set_dim (series->interpolation_props, 0, go_data_scalar_val_new (0.), NULL);
+	gog_dataset_set_dim (series->interpolation_props, 1, go_data_scalar_val_new (0.), NULL);
 }
 
 static void
@@ -1377,6 +1589,11 @@ gog_xy_series_finalize (GObject *obj)
 		series->y_errors = NULL;
 	}
 
+	if (series->interpolation_props != NULL) {
+		g_object_unref (series->interpolation_props);
+		series->interpolation_props = NULL;
+	}
+
 	G_OBJECT_CLASS (series_parent_klass)->finalize (obj);
 }
 
@@ -1462,6 +1679,14 @@ gog_xy_series_set_property (GObject *obj, guint param_id,
 		series->invalid_as_zero = g_value_get_boolean (value);
 		gog_object_request_update (GOG_OBJECT (series));
 		break;
+	case SERIES_PROP_CLAMP0:
+		gog_dataset_set_dim (series->interpolation_props, 0,
+				     go_data_scalar_val_new (g_value_get_double (value)), NULL);
+		break;
+	case SERIES_PROP_CLAMP1:
+		gog_dataset_set_dim (series->interpolation_props, 1,
+				     go_data_scalar_val_new (g_value_get_double (value)), NULL);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -1483,6 +1708,12 @@ gog_xy_series_get_property (GObject *obj, guint param_id,
 	case SERIES_PROP_INVALID_AS_ZERO:
 		g_value_set_boolean (value, series->invalid_as_zero);
 		break;
+	case SERIES_PROP_CLAMP0:
+		g_value_set_double (value, series->clamped_derivs[0]);
+		break;
+	case SERIES_PROP_CLAMP1:
+		g_value_set_double (value, series->clamped_derivs[1]);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -1587,6 +1818,8 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 
 	series_klass->has_interpolation = TRUE;
 	series_klass->has_fill_type	= TRUE;
+	series_klass->series_element_type = GOG_XY_SERIES_ELEMENT_TYPE;
+	series_klass->get_interpolation_params = gog_xy_series_get_interpolation_params;
 
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
@@ -1595,19 +1828,31 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 			_("X error bars"),
 			_("GogErrorBar *"),
 			GOG_ERROR_BAR_TYPE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_YERRORS,
 		g_param_spec_object ("y-errors", 
 			_("Y error bars"),
 			_("GogErrorBar *"),
 			GOG_ERROR_BAR_TYPE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_INVALID_AS_ZERO,
 		g_param_spec_boolean ("invalid-as-zero", 
 			_("Invalid as zero"),
 			_("Replace invalid values by 0 when drawing markers or bubbles"),
 			FALSE, 
-			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, SERIES_PROP_CLAMP0,
+		g_param_spec_double ("clamp0", 
+			_("Clamp at start"),
+			_("Slope at start of the interpolated curve when using clamped spline interpolation"),
+			-G_MAXDOUBLE, G_MAXDOUBLE, 0.,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, SERIES_PROP_CLAMP1,
+		g_param_spec_double ("clamp1", 
+			_("Clamp at end"),
+			_("Slope at end of the interpolated curve when using clamped spline interpolation"),
+			-G_MAXDOUBLE, G_MAXDOUBLE, 0.,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	series_klass->valid_fill_type_list = valid_fill_type_list;
 }
@@ -1627,6 +1872,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_xy_view_register_type (module);
 	gog_xy_series_view_register_type (module);
 	gog_xy_series_register_type (module);
+	gog_xy_series_element_register_type (module);
 }
 
 G_MODULE_EXPORT void
